'use strict';

const dotenv = require('dotenv')
dotenv.config()

const _ = require('underscore');
const requiredParams = [
    'DEFAULT_LANGUAGE',
    'APP_NAME',
    'PORT',
    'CMS_LINK_PATH',
    'CMS_WRITE_PATH',
    'TWILIO_ACCOUNT_SID',
    'TWILIO_AUTH_TOKEN',
    'TWILIO_MOBILE_NO',
    'AWS_ACCESS_KEY_ID',
    'AWS_ACCESS_SECRET_KEY',
    'AWS_REGION',
    'AWS_SES_REGION',
    'AWS_S3_BUCKET_NAME',
    'AWS_S3_URL_PRIFIX',
    'AWS_S3_USER_BUCKET',
    'GOOGLE_API_KEY',
    'DATABASE_URL',
    'FIREBASE_TRACKER_PATH',
    'FIREBASE_SERVICE_ACCOUNT_PATH',
    'AWS_S3_CUSTOMER_BUCKET',
    'STRIPE_TEST_KEY',
    'STRIPE_TEST_KEY',
    'AWS_S3_VEHICLE_TYPE_BUCKET',
    'AWS_S3_VEHICLE_BUCKET',
    'AWS_S3_PROPERTY_BUCKET',
    'AWS_S3_AMENITY_BUCKET',
    'AWS_S3_CHAT_BUCKET',
    'PUSH_SERVER_KEY',
    'BASE_URL',
    'CLICKSEND_USERNAME',
    'CLICKSEND_API_KEY',
];

for (let i = 0; i < requiredParams.length; i++) {
    if (!_.has(process.env, requiredParams[i])) {
        console.log(
            'Error: environment variables have not been properly setup for the Parksie Platform. The variable:',
            requiredParams[i],
            'was not found.'
        );

        throw new Error('Parksie Platform Environment Variables Not Properly Set');
    }
}

const admin = require('firebase-admin');
const trackerConfig = require(process.env.FIREBASE_TRACKER_PATH);
const serviceAccount = require(process.env.FIREBASE_SERVICE_ACCOUNT_PATH);
admin.initializeApp({
    credential: admin.credential.cert(serviceAccount),
    databaseURL: trackerConfig.databaseURL,
});
const firebaseDBRef = admin.database();

module.exports = {
    time_zone: 'Asia/Kolkata',
    default_language: process.env.DEFAULT_LANGUAGE,
    base_url: process.env.BASE_URL,
    appName: process.env.APP_NAME,
    port: process.env.PORT,
    stripeInfo: {
        test_key: process.env.STRIPE_TEST_KEY,
        live_key: process.env.STRIPE_LIVE_KEY,
    },
    cms: {
        link_path: process.env.CMS_LINK_PATH,
        write_path: process.env.CMS_WRITE_PATH
    },
    clicksend: {
        username: process.env.CLICKSEND_USERNAME,
        api_key: process.env.CLICKSEND_API_KEY
    },
    twilio: {
        accountSid: process.env.TWILIO_ACCOUNT_SID,
        authToken: process.env.TWILIO_AUTH_TOKEN,
        mobileNo: process.env.TWILIO_MOBILE_NO
    },
    plaid: {
        // client_id: '613777a496020b0010412ea8',
        client_id: '613777a496020b0010412ea8',
        secret: '8f7d7fedc41d5cef5a754d5c0639dd',
        test_secret: '231787a8dd4b37268cbdc1006acea2',
	// test_secret: '5ac4c8c43108e00d40e37f635d9c68',
    },
    push_server_key: process.env.PUSH_SERVER_KEY,
    firebase: {
        firebaseDBRef: admin.database(),
    },
    aws: {
        keyId: process.env.AWS_ACCESS_KEY_ID,
        key: process.env.AWS_ACCESS_SECRET_KEY,
        region: process.env.AWS_REGION,
        sesRegion: process.env.AWS_SES_REGION,
        bucketName: process.env.AWS_S3_BUCKET_NAME,
        prefix: process.env.AWS_S3_URL_PRIFIX,
        s3: {
            assetUpload: {
                bucket: process.env.AWS_S3_BUCKET_NAME
            },
            userBucket: process.env.AWS_S3_USER_BUCKET,
            customerBucket: process.env.AWS_S3_CUSTOMER_BUCKET,
            vehicleTypeBucket: process.env.AWS_S3_VEHICLE_TYPE_BUCKET,
            vehicleBucket: process.env.AWS_S3_VEHICLE_BUCKET,
            propertyBucket: process.env.AWS_S3_PROPERTY_BUCKET,
            amenityBucket: process.env.AWS_S3_AMENITY_BUCKET,
            chatBucket: process.env.AWS_S3_CHAT_BUCKET,
            dbBucket: process.env.AWS_DB_BUCKET
        },
        mongoDumpUrl: process.env.AWS_MONGO_DUMP_URL,
        mongoRestoreUrl: process.env.AWS_MONGO_RESTORE_URL,
        rootDir: process.env.AWS_ROOT_DIR,
    },
    geocoderInfo: {
        provider: 'google',
        httpAdapter: 'https',
        apiKey: process.env.GOOGLE_API_KEY,
        formatter: null
    },
    google_key: process.env.GOOGLE_API_KEY,
};
