'use strict';

const config = require('./../../config');
const logger = require('./../../utils/logger');
const errors = require('./../../utils/dz-errors-api');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator-api');
const label = require('./../../models/language-label');
const moment = require('moment');
let async = require('async');
let _ = require('underscore');
const S3Handler = require('./../../utils/s3-handler');
const s3Handler = new S3Handler();
const {
    generateString
} = require('./../../utils/random-string-generator');

/*
    Name : Upload Image
    Purpose : Upload Image
    Original Author : Gaurav Patel
    Created At : 28th Sep 2020
*/
const uploadImage = (requestParam, bucketName, done) => {
    let randomStr = generateString(8, true, false, false);
    let fileType = requestParam.name.split('.').pop();
    requestParam['file_name'] = moment().unix() + randomStr + '.' + fileType;

    console.log("requestParam of upload image:-",requestParam)
    console.log("bucketName path of upload image:-",bucketName)
    console.log("fileType of upload image:-",fileType)
    
    s3Handler.upload(requestParam, bucketName, fileType, (error, imageData) => {
        if (error) {
            console.log("error :------",error)
            logger('Error: can not upload image on aws server', '');
            done(errors.internalServer(true), null);
            return;
        }
        console.log(imageData.Location)
        done(null, requestParam.file_name);
    });
};

const removeMultipleImages = (objects, done) => {
    s3Handler.deleteMultipleFiles(objects, config.aws.bucketName, (error, res) => {
        if (error) {
            done(errors.internalServer(true), null);
            return;
        } else {
            done(null, res);
        }
    });
};

/*
    Name : Get Radius Driver
    Purpose : Get Radius Driver
    Original Author : Gaurav Patel
    Created At : 20th Aug 2020
*/
const getRadiusDriver = async(requestParam) => {
    return new Promise(async(resolve, reject) => {
        try {
            var geoQuery = config.firebase.geoDriverRef.query({
                center: [parseFloat(requestParam.pickup_lattitude), parseFloat(requestParam.pickup_longitude)],
                radius: requestParam.radius
            });
            let driverIds = []
            geoQuery.on("key_entered", (key, location, distance) => {
                driverIds.push(key)
            });
            geoQuery.on("ready", (key, location, distance) => {
                resolve(driverIds);
                return;
            });
        } catch (error) {
            reject(error)
            return;
        }
    })
};

module.exports = {
    uploadImage,
    removeMultipleImages,
    getRadiusDriver
};