'use strict';

const config = require('./../../config');
const logger = require('./../../utils/logger');
const errors = require('./../../utils/dz-errors-api');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator-api');
let async = require('async');
let _ = require('underscore');
let moment = require('moment');
var days = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'];
var d = new Date();
var dayName = days[d.getDay()]
const { forEach } = require('p-iteration');
const notificationHandler = require('./../../model_handlers/api/notification-handler');
const timeZone = require('moment-timezone');
const LD = require('lodash');
const { sendSMS } = require('./../../utils/sms-manager');
const { createReservationChargesStripe, stripePaymentConfirmed } = require('../../helpers/payments');
const distance = require('google-distance');
distance.apiKey = config.google_key;

/*
    Name : List
    Purpose : List
    Original Author : Gaurav Patel
    Created At : 18th Mar 2021
*/
const parkingSpotList = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            console.log("requestParam",requestParam)
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let sortColumn = {
                created_at: -1
            }
            if (requestParam.sort_by) {
                if (requestParam.sort_by == 'price') {
                    sortColumn = {
                        'pricing.hourly_rate': 1
                    }
                }
                if (requestParam.sort_by == 'distance') {
                    sortColumn = {
                        created_at: -1
                    }
                }
            }

            let columnAndValues = {
                status: 'completed',
                discoverable_status: 'on',
                available_days: { $in: [dayName] }
            }
            if (requestParam.vehicle_type_id && requestParam.vehicle_type_id != '') {
                columnAndValues['parking_size.vehicle_type_id'] = requestParam.vehicle_type_id
            }
            let to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone).format('YYYY-MM-DD')
            let from_date = timeZone(new Date(requestParam.from_date)).tz(requestParam.time_zone).format('YYYY-MM-DD')
            columnAndValues['$or'] = [{
                is_specific_date_range: true
            }, {
                is_specific_date_range: false,
                from_date_range: {
                    $lte: new Date(to_date + 'T23:59:59.000Z'),
                },
                to_date_range: {
                    $gte: new Date(from_date + 'T00:00:00.000Z')
                }
            }]
            console.log("qwerty:----------",requestParam)
            if (requestParam.filter_by_amenity) {
                
                columnAndValues.parking_spot_amenity_id = { $in: requestParam.filter_by_amenity.split(',') }
                console.log("aminities here:- ",columnAndValues.parking_spot_amenity_id)
            }
            if (requestParam.filter_by_spot_type) {
                columnAndValues.parking_spot_type_id = { $in: requestParam.filter_by_spot_type.split(',') }
            }
            if (requestParam.filter_by_coverage) {
                columnAndValues.parking_spot_coverage_id = { $in: requestParam.filter_by_coverage.split(',') }
            }
            if (requestParam.filter_by_terrain) {
                columnAndValues.parking_spot_terrain_id = { $in: requestParam.filter_by_terrain.split(',') }
            }
            // columnAndValues['$or'] = [{
            //     'reservationDetails.status': {$ne:'completed'}
            // }, {
            //     'reservationDetails.status': {$ne:'cancelled'}
            // }]
            //columnAndValues['reservationDetails.status'] = {$ne:'completed'}
            let joinArr = [{
                $geoNear: {
                    includeLocs: "location",
                    distanceField: "distance",
                    near: { type: 'Point', coordinates: [parseFloat(requestParam.longitude), parseFloat(requestParam.latitude)] },
                    maxDistance: 100 * 1000,
                    spherical: true
                }
            }, {
                $lookup: {
                    from: 'reservations',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'reservationDetails'
                }
            }, {
                $lookup: {
                    from: 'parking_spot_types',
                    localField: 'parking_spot_type_id',
                    foreignField: 'parking_spot_type_id',
                    as: 'spotTypesDetails'
                }
            }, {
                $unwind: "$spotTypesDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_terrains',
                    localField: 'parking_spot_terrain_id',
                    foreignField: 'parking_spot_terrain_id',
                    as: 'spotTerrainDetails'
                }
            }, {
                $unwind: "$spotTerrainDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_coverages',
                    localField: 'parking_spot_coverage_id',
                    foreignField: 'parking_spot_coverage_id',
                    as: 'spotCoverageDetails'
                }
            }, {
                $unwind: "$spotCoverageDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_amenities',
                    localField: 'parking_spot_amenity_id',
                    foreignField: 'parking_spot_amenity_id',
                    as: 'spotAmenityDetails'
                }
            }, {
                $lookup: {
                    from: 'access_instructions',
                    localField: 'access_instruction_id',
                    foreignField: 'access_instruction_id',
                    as: 'accessDetails'
                }
            }, {
                $unwind: "$accessDetails"
            }, {
                $lookup: {
                    from: 'property_ratings',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'ratingDetails'
                }
            }, {
                $match: columnAndValues
            }, {
                $sort: sortColumn
            }, {
                $project: {
                    _id: 0,
                    property_id: 1,
                    status: 1,
                    location: 1,
                    contact_details: 1,
                    property_type: 1,
                    entrance_longitude: 1,
                    latitude: 1,
                    longitude: 1,
                    address: 1,
                    entrance_latitude: 1,
                    entrance_property: 1,
                    customer_id: 1,
                    parking_spot_type_id: 1,
                    parking_spot_type_title: "$spotTypesDetails.title." + requestParam.code,
                    parking_spot_coverage_id: 1,
                    parking_spot_coverage_title: "$spotCoverageDetails.title." + requestParam.code,
                    parking_spot_terrain_id: 1,
                    parking_spot_terrain_title: "$spotTerrainDetails.title." + requestParam.code,
                    parking_spot_amenity_id: 1,
                    parking_spot_amenity_details: "$spotAmenityDetails",
                    access_instruction_id: 1,
                    access_instruction_title: "$accessDetails.title." + requestParam.code,
                    parking_spot_description: 1,
                    parking_spot_photos: 1,
                    from_date_range: 1,
                    to_date_range: 1,
                    available_days: 1,
                    from_reservation_time: 1,
                    to_reservation_time: 1,
                    from_entry_time: 1,
                    to_entry_time: 1,
                    is_auto_activate: 1,
                    distance_from_parking_spot: 1,
                    private_instruction: 1,
                    geo_tag: 1,
                    parking_size: 1,
                    pricing: 1,
                    grace_period: 1,
                    ratings: "$ratingDetails",
                    //reservations:"$reservationDetails",
                    reservations: {
                        $filter: {
                            input: "$reservationDetails",
                            as: "fav",
                            cond: {
                                $or: [
                                    { $ne: ['$$fav.status', 'completed'] },
                                    { $ne: ['$$fav.status', 'cancelled'] }
                                ]
                            }
                        }
                    },
                }
            }];
            let property = await query.joinWithAnd(dbConstants.dbSchema.properties, joinArr);
            property = JSON.parse(JSON.stringify(property))
            let mainArr = [];
            async.forEachSeries(property, async function (elem, Callback_s1) {

                distance.get(
                    {
                        index: 1,
                        origin: '' + requestParam.latitude + ',' + requestParam.longitude + '',
                        destination: '' + elem.latitude + ',' + elem.longitude + ''
                    },
                    async function (err, data) {
                        if (err) {
                            reject(errors.googleDistanceNotFound(true, requestParam.code));
                            return;
                        }
                        elem.distance_from_parking_spot = (data.distanceValue / 1000) * 0.6214

                        let rating = 0;
                        if (elem.ratings.length > 0) {
                            rating = LD.sumBy(elem.ratings, 'rating')
                            rating = parseFloat(rating) / elem.ratings.length;
                        }
                        elem.rating = parseFloat(rating).toFixed(1);
                        elem.out_of_rating = elem.ratings.length;
                        delete elem.ratings

                        let arr = [];
                        elem.is_favorite = false;
                        let fav = await query.selectWithAndOne(dbConstants.dbSchema.seeker_favourites, { customer_id: requestParam.customer_id, property_id: elem.property_id }, { _id: 0, favourite_id: 1 });
                        if (fav) {
                            elem.is_favorite = true;
                        }

                        _.each(elem.parking_spot_photos, (rec) => {
                            if (rec != '') {
                                arr.push(config.aws.prefix + config.aws.s3.propertyBucket + '/' + rec)
                            }
                        });
                        _.each(elem.parking_spot_amenity_details, (item) => {
                            item.title = item.title[requestParam.code]
                            item.icon = item.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + item.icon : '';
                            delete item._id
                            delete item.created_at
                            delete item.updated_at
                            delete item.__v
                            delete item.status
                        });
                        elem.parking_spot_photos = arr

                        elem.grace_period = elem.grace_period ? elem.grace_period : 10

                        let is_free = true;
                        requestParam.from_date = timeZone(new Date(requestParam.from_date)).tz(requestParam.time_zone);
                        requestParam.to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
                        await forEach(elem.reservations, async (rec) => {
                            let start_date = timeZone(new Date(rec.from_date)).tz(requestParam.time_zone);
                            let last_date = timeZone(new Date(rec.to_date)).tz(requestParam.time_zone);
                            last_date = moment(last_date).add(elem.grace_period, 'minutes');
                            if (moment(start_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]') || moment(last_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]')) {
                                is_free = false
                            }
                        });
                        delete elem.reservations
                        delete elem.grace_period
                        if (is_free) {
                            mainArr.push(elem)
                        }
                        Callback_s1();
                    });
            }, function () {
                if (requestParam.sort_by) {
                    if (requestParam.sort_by == 'distance') {
                        mainArr = _.sortBy(mainArr, 'distance_from_parking_spot')
                    }
                }
                resolve(mainArr)
                return;
            });
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};
// const parkingSpotList = async(requestParam) => {
//     return new Promise(async(resolve, reject) => {
//         try {
//             let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, {customer_id: requestParam.customer_id}, { _id: 0, customer_id: 1} );
//             if(!response){
//                 reject(errors.userNotFound(true, requestParam.code));
//                 return;
//             }
//             let sortColumn = {
//                 created_at: -1
//             }
//             if(requestParam.sort_by){
//                 if(requestParam.sort_by == 'price'){
//                     sortColumn = {
//                         'pricing.hourly_rate': -1
//                     }
//                 }
//                 if(requestParam.sort_by == 'distance'){
//                     sortColumn = {
//                         created_at: -1
//                     }
//                 }
//             }

//             let columnAndValues = {
//                 status:'completed',
//                 'parking_size.vehicle_type_id': requestParam.vehicle_type_id,
//                 available_days: {$in: [dayName]}
//             }
//             columnAndValues['$or'] = [{
//                 is_specific_date_range: false
//             }, {
//                 is_specific_date_range: true,
//                 from_date_range: {
//                     $lte: new Date(requestParam.to_date+'T23:59:59.000Z'),
//                 },
//                 to_date_range: {
//                     $gte: new Date(requestParam.from_date+'T00:00:00.000Z')
//                 }
//             }]
//             if(requestParam.filter_by_amenity){
//                 columnAndValues.parking_spot_amenity_id = {$in: requestParam.filter_by_amenity.split(',')}
//             }
//             if(requestParam.filter_by_spot_type){
//                 columnAndValues.parking_spot_type_id = {$in: requestParam.filter_by_spot_type.split(',')}
//             }
//             if(requestParam.filter_by_coverage){
//                 columnAndValues.parking_spot_coverage_id = {$in: requestParam.filter_by_coverage.split(',')}
//             }
//             if(requestParam.filter_by_terrain){
//                 columnAndValues.parking_spot_terrain_id = {$in: requestParam.filter_by_terrain.split(',')}
//             }
//             let joinArr = [{
//                 $geoNear: {
//                     includeLocs: "location",
//                     distanceField: "distance",
//                     near: {type: 'Point', coordinates: [parseFloat(requestParam.longitude), parseFloat(requestParam.latitude)]},
//                     maxDistance: 100 * 1000,
//                     spherical: true
//                 }
//             }, {
//                 $lookup: {
//                     from: 'parking_spot_types',
//                     localField: 'parking_spot_type_id',
//                     foreignField: 'parking_spot_type_id',
//                     as: 'spotTypesDetails'
//                 }
//             },  {
//                 $unwind: "$spotTypesDetails"
//             },  {
//                 $lookup: {
//                     from: 'parking_spot_terrains',
//                     localField: 'parking_spot_terrain_id',
//                     foreignField: 'parking_spot_terrain_id',
//                     as: 'spotTerrainDetails'
//                 }
//             },  {
//                 $unwind: "$spotTerrainDetails"
//             },  {
//                 $lookup: {
//                     from: 'parking_spot_coverages',
//                     localField: 'parking_spot_coverage_id',
//                     foreignField: 'parking_spot_coverage_id',
//                     as: 'spotCoverageDetails'
//                 }
//             },  {
//                 $unwind: "$spotCoverageDetails"
//             },  {
//                 $lookup: {
//                     from: 'parking_spot_amenities',
//                     localField: 'parking_spot_amenity_id',
//                     foreignField: 'parking_spot_amenity_id',
//                     as: 'spotAmenityDetails'
//                 }
//             },  {
//                 $lookup: {
//                     from: 'access_instructions',
//                     localField: 'access_instruction_id',
//                     foreignField: 'access_instruction_id',
//                     as: 'accessDetails'
//                 }
//             },  {
//                 $unwind: "$accessDetails"
//             },  { 
//                 $match : columnAndValues
//             }, { 
//                 $sort : sortColumn
//             }, {
//                 $project: {
//                     _id: 0,
//                     property_id: 1,
//                     status: 1,
//                     location: 1,
//                     contact_details: 1,
//                     property_type: 1,
//                     entrance_longitude: 1,
//                     latitude: 1,
//                     longitude: 1,
//                     address: 1,
//                     entrance_latitude: 1,
//                     entrance_property: 1,
//                     customer_id: 1,
//                     parking_spot_type_id: 1,
//                     parking_spot_type_title: "$spotTypesDetails.title."+requestParam.code,
//                     parking_spot_coverage_id:1,
//                     parking_spot_coverage_title: "$spotCoverageDetails.title."+requestParam.code,
//                     parking_spot_terrain_id:1,
//                     parking_spot_terrain_title: "$spotTerrainDetails.title."+requestParam.code,
//                     parking_spot_amenity_id:1,
//                     parking_spot_amenity_details:"$spotAmenityDetails",
//                     access_instruction_id:1,
//                     access_instruction_title:"$accessDetails.title."+requestParam.code,
//                     parking_spot_description:1,
//                     parking_spot_photos:1,
//                     from_date_range:1,
//                     to_date_range:1,
//                     available_days:1,
//                     from_reservation_time:1,
//                     to_reservation_time:1,
//                     from_entry_time:1,
//                     to_entry_time:1,
//                     is_auto_activate:1,
//                     distance_from_parking_spot:1,
//                     private_instruction:1,
//                     geo_tag:1,
//                     parking_size:1,
//                     pricing:1,
//                 }
//             }];
//             let property = await query.joinWithAnd(dbConstants.dbSchema.properties, joinArr);
//             property = JSON.parse(JSON.stringify(property))
//             await forEach(property, async(elem) => {
//                 let arr = [];
//                 elem.rating = 1.2;
//                 elem.out_of_rating = 5;
//                 elem.is_favorite = false;
//                 let fav = await query.selectWithAndOne(dbConstants.dbSchema.seeker_favourites, {customer_id: requestParam.customer_id, property_id:elem.property_id}, { _id: 0, favourite_id: 1} );
//                 if(fav){
//                     elem.is_favorite = true;
//                 }

//                 _.each(elem.parking_spot_photos, (rec) => {
//                     if(rec != ''){
//                         arr.push(config.aws.prefix + config.aws.s3.propertyBucket + '/' + rec)
//                     }
//                 });
//                 _.each(elem.parking_spot_amenity_details, (item) => {
//                     item.title = item.title[requestParam.code]
//                     item.icon = item.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + item.icon : '';
//                     delete item._id
//                     delete item.created_at
//                     delete item.updated_at
//                     delete item.__v
//                     delete item.status
//                 });
//                 elem.parking_spot_photos = arr
//             });
//             resolve(property)
//             return;
//         } catch (error) {
//             console.log(error)
//             reject(error)
//             return
//         }
//     })
// };

/*
    Name : Favourite/Unfavourite
    Purpose : Favourite/Unfavourite
    Original Author : Gaurav Patel
    Created At : 05th APr 2021
*/
const favourite = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let favourite = await query.selectWithAndOne(dbConstants.dbSchema.seeker_favourites, { customer_id: requestParam.customer_id, property_id: requestParam.property_id }, { _id: 0, favourite_id: 1, property_id: 1 });
            if (favourite) {
                await query.removeMultiple(dbConstants.dbSchema.seeker_favourites, { customer_id: requestParam.customer_id, property_id: requestParam.property_id })
            }
            else {
                await query.insertSingle(dbConstants.dbSchema.seeker_favourites, requestParam);
            }
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : Favourite List
    Purpose : Favourite List
    Original Author : Gaurav Patel
    Created At : 05th APr 2021
*/
const favouriteList = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let favourite = await query.selectWithAnd(dbConstants.dbSchema.seeker_favourites, { customer_id: requestParam.customer_id }, { _id: 0, favourite_id: 1, property_id: 1 });
            let ids = _.pluck(favourite, 'property_id')
            let columnAndValues = {
                property_id: { $in: ids }
            }
            let joinArr = [{
                $lookup: {
                    from: 'parking_spot_types',
                    localField: 'parking_spot_type_id',
                    foreignField: 'parking_spot_type_id',
                    as: 'spotTypesDetails'
                }
            }, {
                $unwind: "$spotTypesDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_terrains',
                    localField: 'parking_spot_terrain_id',
                    foreignField: 'parking_spot_terrain_id',
                    as: 'spotTerrainDetails'
                }
            }, {
                $unwind: "$spotTerrainDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_coverages',
                    localField: 'parking_spot_coverage_id',
                    foreignField: 'parking_spot_coverage_id',
                    as: 'spotCoverageDetails'
                }
            }, {
                $unwind: "$spotCoverageDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_amenities',
                    localField: 'parking_spot_amenity_id',
                    foreignField: 'parking_spot_amenity_id',
                    as: 'spotAmenityDetails'
                }
            }, {
                $lookup: {
                    from: 'access_instructions',
                    localField: 'access_instruction_id',
                    foreignField: 'access_instruction_id',
                    as: 'accessDetails'
                }
            }, {
                $unwind: "$accessDetails"
            }, {
                $lookup: {
                    from: 'property_ratings',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'ratingDetails'
                }
            }, {
                $match: columnAndValues
            }, {
                $sort: { created_at: -1 }
            }, {
                $project: {
                    _id: 0,
                    property_id: 1,
                    status: 1,
                    location: 1,
                    contact_details: 1,
                    property_type: 1,
                    entrance_longitude: 1,
                    latitude: 1,
                    longitude: 1,
                    address: 1,
                    entrance_latitude: 1,
                    entrance_property: 1,
                    customer_id: 1,
                    parking_spot_type_id: 1,
                    parking_spot_type_title: "$spotTypesDetails.title." + requestParam.code,
                    parking_spot_coverage_id: 1,
                    parking_spot_coverage_title: "$spotCoverageDetails.title." + requestParam.code,
                    parking_spot_terrain_id: 1,
                    parking_spot_terrain_title: "$spotTerrainDetails.title." + requestParam.code,
                    parking_spot_amenity_id: 1,
                    parking_spot_amenity_details: "$spotAmenityDetails",
                    access_instruction_id: 1,
                    access_instruction_title: "$accessDetails.title." + requestParam.code,
                    parking_spot_description: 1,
                    parking_spot_photos: 1,
                    from_date_range: 1,
                    to_date_range: 1,
                    available_days: 1,
                    from_reservation_time: 1,
                    to_reservation_time: 1,
                    from_entry_time: 1,
                    to_entry_time: 1,
                    is_auto_activate: 1,
                    distance_from_parking_spot: 1,
                    private_instruction: 1,
                    geo_tag: 1,
                    parking_size: 1,
                    pricing: 1,
                    ratings: "$ratingDetails",
                }
            }];
            let property = await query.joinWithAnd(dbConstants.dbSchema.properties, joinArr);
            property = JSON.parse(JSON.stringify(property))
            _.each(property, (elem) => {
                let arr = [];
                let rating = 0;
                if (elem.ratings.length > 0) {
                    rating = LD.sumBy(elem.ratings, 'rating')
                    rating = parseFloat(rating) / elem.ratings.length;
                }
                elem.rating = parseFloat(rating).toFixed(1);
                elem.out_of_rating = elem.ratings.length;
                delete elem.ratings

                _.each(elem.parking_spot_photos, (rec) => {
                    if (rec != '') {
                        arr.push(config.aws.prefix + config.aws.s3.propertyBucket + '/' + rec)
                    }
                });
                _.each(elem.parking_spot_amenity_details, (item) => {
                    item.title = item.title[requestParam.code]
                    item.icon = item.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + item.icon : '';
                    delete item._id
                    delete item.created_at
                    delete item.updated_at
                    delete item.__v
                    delete item.status
                });
                elem.parking_spot_photos = arr
            });
            resolve(property)
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : reservation
    Purpose : reservation
    Original Author : Gaurav Patel
    Created At : 05th APr 2021
*/
const reservation = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            console.log(">>>>>>>>",requestParam)
            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1, stripe_profile_id: 1, email: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            console.log("response",response)
            let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: requestParam.property_id }, { _id: 0, property_id: 1, customer_id: 1, address: 1, grace_period: 1 });
            if (!property) {
                reject(errors.propertyNotFound(true, requestParam.code));
                return;
            }
            let getObj = {}
            getObj['$or'] = [{
                status: 'reserved'
            }, {
                status: 'parked'
            }]
            getObj.property_id = requestParam.property_id
            let reservations = await query.selectWithAnd(dbConstants.dbSchema.reservations, getObj, { _id: 0 });

            property.grace_period = property.grace_period ? property.grace_period : 10;
            let is_free = true;
            let is_instant_booked = false;

            requestParam.from_date = new Date(requestParam.from_date);
            requestParam.to_date = new Date(requestParam.to_date);
            console.log("before forLoop", requestParam)
            await forEach(reservations, async (rec) => {
                let start_date = timeZone(new Date(rec.from_date)).tz(requestParam.time_zone);
                let last_date = timeZone(new Date(rec.to_date)).tz(requestParam.time_zone);
                last_date = moment(last_date).add(property.grace_period, 'minutes');
                if (moment(start_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]') || moment(last_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]')) {
                    is_free = false
                }

                let end_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
                let first_end_date = moment(end_date).add(1, 'minutes');
                let second_end_date = moment(end_date).add(5, 'minutes');
                if (moment(start_date).isBetween(first_end_date, second_end_date, null, '[]')) {
                    is_instant_booked = true
                }
            });

            if (!is_free) {
                reject(errors.alreadyReservedSlot(true, requestParam.code));
                return;
            }
            console.log("after loop")
            // for same vehicle & same time
            let is_vehicle_reserved = false
            delete getObj.property_id
            getObj.vehicle_id = requestParam.vehicle_id
            console.log("GetObject here ::::::====>",getObj)
            reservations = await query.selectWithAnd(dbConstants.dbSchema.reservations, getObj, { _id: 0 });
            console.log(">>>>>>>>>> again loop",reservations)
            await forEach(reservations, async (rec) => {
                let start_date = timeZone(new Date(rec.from_date)).tz(requestParam.time_zone);
                let last_date = timeZone(new Date(rec.to_date)).tz(requestParam.time_zone);
                if (moment(start_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]') || moment(last_date).isBetween(requestParam.from_date, requestParam.to_date, null, '[]')) {
                    if (rec.vehicle_id == requestParam.vehicle_id) {
                        is_vehicle_reserved = true
                    }
                }
            });
            
            if (is_vehicle_reserved) {
                reject(errors.alreadyReservedVehicle(true, requestParam.code));
                return;
            }
            
            /**
             * New payment logics started from here
             */
            
            console.log("Denish Log customer_id->",property.customer_id)
            // console.log("Denish Log ne->",$ne)

            let ownerDetails = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: property.customer_id, stripe_connected_account_id: { $ne: "" } }, { _id: 0, bank_details_added: 1, bank_details: 1, stripe_connected_account_id: 1 });
            console.log("Denish Log ownerDetails->",ownerDetails)
            if (!ownerDetails) {
                //code here
                console.log("ownerDetails here---------")
                reject({ error: "Owner not found!" });
                return;
            }

            if (ownerDetails.bank_details_added == false) {
                console.log("---------")
                reject(errors.bankDetailsNotAddad(true, requestParam.code));
                return;
            }
            console.log(">>>>>>>>>>>>>>>><<<<<<<>>>>>",ownerDetails)
            
            console.log("Data Price->",requestParam.price)
            console.log("Data cardID->",requestParam.card_id)
            console.log("Data Profile ID->",response.stripe_profile_id)
            console.log("Data Account ID->",ownerDetails.stripe_connected_account_id)
            
            let { error, stripeRes, gateway_charge, company_earning,
                owner_earning } = await createReservationChargesStripe({
                    settings,
                    is_payment_confirmed: false,
                    amount: requestParam.price,
                    card_id: requestParam.card_id,
                    customer_stripe_id: response.stripe_profile_id,
                    owner_connected_account: ownerDetails.stripe_connected_account_id
                })
            if (error) {
                // error code here
                console.log(error);
                reject(error)
                return;
            }
            console.log("Denish Log 8")
            console.log(error, stripeRes, gateway_charge, company_earning, owner_earning);
            requestParam.transaction_id = stripeRes.id
            requestParam.company_earn = company_earning
            requestParam.owner_earn = owner_earning

            /** New payment logics ended here */

            // for same vehicle & same time

            // let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);
            // let stripeRes = await (new Promise(async(resolve1, reject1) => {
            //     let chargeAmount = Math.round(parseFloat(requestParam.price) * 100).toFixed(2);
            //     stripe.charges.create({
            //         amount: parseFloat(chargeAmount),
            //         currency: "usd",
            //         source: requestParam.card_id,
            //         customer: response.stripe_profile_id,
            //         description: response.email,
            //         capture: false
            //     }, async function(err, charge) {
            //         if(err){
            //             reject(errors.stripeError(true, err.message,requestParam.code));
            //             return;
            //         }
            //         resolve1(charge)
            //     })
            // }));

            // let newPrice = Number(requestParam.price);
            // newPrice = (newPrice * 2.9 / 100) + 0.30;
            // newPrice = Number(requestParam.price) - newPrice;
            // requestParam.transaction_id = stripeRes.id
            console.log("Denish Log 9")
            requestParam.owner_id = property.customer_id;
            requestParam.driver_details = JSON.parse(`${requestParam.driver_details}`)

            // requestParam.company_earn = ((parseFloat(newPrice)) * (settings.company_percentage ? settings.company_percentage : 10)) / 100
            // requestParam.owner_earn = parseFloat(newPrice) - parseFloat(requestParam.company_earn)

            requestParam.company_percentage = settings.company_percentage
            console.log("Parking ReqParam->",requestParam)
            let reservation = await query.insertSingle(dbConstants.dbSchema.reservations, requestParam);

            sendOwnerNotificaton({ reservation_id: reservation.reservation_id, owner_id: requestParam.owner_id, customer_id: requestParam.customer_id, push_code: 'BOOK_RESERVATION_TO_OWNER', code: requestParam.code, address: property.address });
            console.log("Denish Log 10")
            let date = timeZone(new Date(reservation.from_date)).tz(requestParam.time_zone).format('lll');
            sendSeekerNotificaton({ reservation_id: reservation.reservation_id, date: date, customer_id: requestParam.customer_id, push_code: 'BOOK_RESERVATION_TO_SEEKER', code: requestParam.code, address: property.address, is_instant_booked: is_instant_booked });
            resolve({ reservation_id: reservation.reservation_id, status: reservation.status })
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const sendOwnerNotificaton = (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let customer = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, name: 1 });
            let owner = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.owner_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: requestParam.push_code }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value[requestParam.code];
                msg = msg.replace('#NAME#', customer ? customer.name : '')
                notificationHandler.sendNotification({
                    message: msg,
                    type: 'book_reservation',
                    title: config.appName,
                    user_id: requestParam.owner_id,
                    device_token: owner ? [owner.device_token] : [],
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: 'owner',
                    reservation_status: 'reserved',
                });
                notificationHandler.setLogs({
                    user_id: [requestParam.owner_id],
                    user_type: 'owner',
                    message: template.caption_value[requestParam.code],
                    description: msg,
                    type: 'book_reservation',
                    address: requestParam.address,
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    reservation_status: 'reserved',
                });
            }
            return false;
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

const sendSeekerNotificaton = (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let customer = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: requestParam.push_code }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value[requestParam.code];
                msg = msg.replace('#DATE#', requestParam.date)
                notificationHandler.sendNotification({
                    message: msg,
                    type: 'book_reservation',
                    title: config.appName,
                    user_id: requestParam.customer_id,
                    device_token: customer ? [customer.device_token] : [],
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: 'seeker',
                    reservation_status: 'reserved',
                });
                notificationHandler.setLogs({
                    user_id: [requestParam.customer_id],
                    user_type: 'seeker',
                    message: template.caption_value[requestParam.code],
                    description: msg,
                    type: 'book_reservation',
                    address: requestParam.address,
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    reservation_status: 'reserved',
                });
            }

            if (requestParam.is_instant_booked && requestParam.is_instant_booked == true) {
                template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: 'SOMEONE_RESERVED_CAN_NOT_EXTEND_PARKING' }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
                if (template) {
                    let msg1 = template.value[requestParam.code];
                    notificationHandler.sendNotification({
                        message: msg1,
                        type: 'someone_reserved',
                        title: config.appName,
                        user_id: requestParam.customer_id,
                        device_token: customer ? [customer.device_token] : [],
                        sender_id: requestParam.customer_id,
                        reservation_id: requestParam.reservation_id,
                        user_type: 'seeker',
                        reservation_status: 'reserved',
                    });
                    notificationHandler.setLogs({
                        user_id: [requestParam.customer_id],
                        user_type: 'seeker',
                        message: template.caption_value[requestParam.code],
                        description: msg1,
                        type: 'someone_reserved',
                        address: requestParam.address,
                        sender_id: requestParam.customer_id,
                        reservation_id: requestParam.reservation_id,
                        reservation_status: 'reserved',
                    });
                }
            }
            return false;
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

/*
    Name : reservation list
    Purpose : reservation list
    Original Author : Gaurav Patel
    Created At : 05th APr 2021
*/
const reservationList = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let matchColumn = {
                customer_id: requestParam.customer_id,
            }
            let today = timeZone(new Date()).tz(requestParam.time_zone).format('YYYY-MM-DD')
            if (requestParam.type == 'current') {
                matchColumn.status = { $in: ['parked'] }
            }
            if (requestParam.type == 'upcoming') {
                matchColumn.status = { $in: ['reserved'] }
            }
            if (requestParam.type == 'past') {
                matchColumn['$or'] = [{
                    status: 'completed'
                }, {
                    status: 'cancelled'
                }]
            };
            let joinArr = [{
                $lookup: {
                    from: 'properties',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'propDetails'
                }
            }, {
                $unwind: "$propDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_types',
                    localField: 'propDetails.parking_spot_type_id',
                    foreignField: 'parking_spot_type_id',
                    as: 'spotTypesDetails'
                }
            }, {
                $unwind: "$spotTypesDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_coverages',
                    localField: 'propDetails.parking_spot_coverage_id',
                    foreignField: 'parking_spot_coverage_id',
                    as: 'spotCoverageDetails'
                }
            }, {
                $unwind: "$spotCoverageDetails"
            }, {
                $lookup: {
                    from: 'customers',
                    localField: 'owner_id',
                    foreignField: 'customer_id',
                    as: 'ownerDetails'
                }
            }, {
                "$unwind": {
                    "path": "$ownerDetails",
                    "preserveNullAndEmptyArrays": true
                }
            }, {
                $lookup: {
                    from: 'property_ratings',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'ratingDetails'
                }
            }, {
                $lookup: {
                    from: 'chats',
                    localField: 'reservation_id',
                    foreignField: 'room',
                    as: 'chatDetails'
                }
            }, {
                "$unwind": {
                    "path": "$chatDetails",
                    "preserveNullAndEmptyArrays": true
                }
            }, {
                $match: matchColumn
            }, {
                $sort: { created_at: -1 }
            }, {
                $project: {
                    _id: 0,
                    reservation_id: 1,
                    property_id: 1,
                    owner_id: 1,
                    status: 1,
                    from_date: 1,
                    to_date: 1,
                    parked_date: 1,
                    price: 1,
                    book_type: 1,
                    parking_spot_photos: "$propDetails.parking_spot_photos",
                    pricing: "$propDetails.pricing",
                    geo_tag: "$propDetails.geo_tag",
                    address: "$propDetails.address",
                    latitude: "$propDetails.latitude",
                    longitude: "$propDetails.longitude",
                    display_date: "",
                    parking_spot_type_title: "$spotTypesDetails.title." + requestParam.code,
                    parking_spot_coverage_title: "$spotCoverageDetails.title." + requestParam.code,
                    owner: "$ownerDetails",
                    ratings: "$ratingDetails",
                    is_chat: "$chatDetails",
                }
            }];
            let reservation = await query.joinWithAnd(dbConstants.dbSchema.reservations, joinArr);
            reservation = JSON.parse(JSON.stringify(reservation))
            //let newArr = [];
            // if(requestParam.type == 'current'){
            //     _.each(reservation, (elem) => {
            //         if(elem.status == 'parked'){
            //             newArr.push(elem)
            //         }
            //         else{
            //             let from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('YYYY-MM-DD')
            //             if(today == from_date){
            //                 newArr.push(elem)
            //             }
            //         }
            //     })
            //     reservation = newArr
            // }
            // if(requestParam.type == 'upcoming'){
            //     _.each(reservation, (elem) => {
            //         if(elem.status == 'reserved'){
            //             let from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('YYYY-MM-DD')
            //             if(today != from_date){
            //                 newArr.push(elem)
            //             }
            //         }
            //     })
            //     reservation = newArr
            // }
            await forEach(reservation, async (elem) => {

                let arr = []
                _.each(elem.parking_spot_photos, (rec) => {
                    if (rec != '') {
                        arr.push(config.aws.prefix + config.aws.s3.propertyBucket + '/' + rec)
                    }
                });
                elem.parking_spot_photos = arr

                // let rating = 0;
                // if(elem.ratings.length > 0){
                //     rating = LD.sumBy(elem.ratings, 'rating')
                //     rating = parseFloat(rating) / elem.ratings.length;
                // }
                // elem.rating = parseFloat(rating).toFixed(1);
                let reservationRatingData = _.findWhere(elem.ratings, { reservation_id: elem.reservation_id });
                elem.rating = (reservationRatingData && reservationRatingData.rating) ? parseFloat(reservationRatingData.rating).toFixed(1) : 0;
                elem.out_of_rating = elem.ratings.length;
                delete elem.ratings

                if (requestParam.type != 'past') {
                    if (elem.status == 'reserved') {
                        elem.display_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('lll')
                    }
                    if (elem.status == 'parked') {
                        let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
                        let to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone);
                        let duration = moment.duration(to_date.diff(todayDate));
                        if (elem.book_type == 'parking_meter') {
                            let from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone);
                            duration = moment.duration(todayDate.diff(from_date));
                        }
                        const hours = parseInt(duration.asHours());
                        const minutes = parseInt(duration.asMinutes()) - hours * 60;
                        if (hours > 0) {
                            elem.display_date = hours + "hrs " + minutes + "mins"
                        }
                        else {
                            elem.display_date = minutes + "mins"
                        }
                    }
                }
                elem.from_time = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('lll')
                elem.to_time = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone).format('lll')

                if (elem.owner) {
                    elem.owner = {
                        customer_id: elem.owner.customer_id,
                        name: elem.owner.name,
                        mobile_country_code: elem.owner.mobile_country_code,
                        mobile: elem.owner.mobile,
                        profile_picture: elem.owner.profile_picture != '' ? config.aws.prefix + config.aws.s3.customerBucket + '/' + elem.owner.profile_picture : ''
                    }
                }
                else {
                    elem.owner = {}
                }

                elem.is_property_fav = false;
                elem.is_property_rated = false;
                elem.is_owner_rated = false;
                let property_rate = await query.selectWithAndOne(dbConstants.dbSchema.property_ratings, { customer_id: requestParam.customer_id, property_id: elem.property_id, reservation_id: elem.reservation_id }, { _id: 0, property_rating_id: 1 });
                if (property_rate) {
                    elem.is_property_rated = true;
                }
                let owner_rate = await query.selectWithAndOne(dbConstants.dbSchema.owner_ratings, { customer_id: requestParam.customer_id, owner_id: elem.owner_id, reservation_id: elem.reservation_id }, { _id: 0, rating_id: 1 });
                if (owner_rate) {
                    elem.is_owner_rated = true;
                }
                let property_fav = await query.selectWithAndOne(dbConstants.dbSchema.seeker_favourites, { customer_id: requestParam.customer_id, property_id: elem.property_id }, { _id: 0, favourite_id: 1 });
                if (property_fav) {
                    elem.is_property_fav = true;
                }
                delete elem.from_date
                delete elem.to_date
                delete elem.parked_date

                let is_chat = 'read';
                if (elem.is_chat) {
                    _.each(elem.is_chat.chat, (rec) => {
                        if (rec.opponent_user_id == requestParam.customer_id) {
                            if (rec.status.includes(requestParam.customer_id) == false) {
                                is_chat = 'unread'
                            }
                        }
                    })
                }
                elem.is_chat = is_chat
            })
            resolve(reservation)
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : reservation details
    Purpose : reservation details
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const reservationDetails = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1, name: 1, mobile_country_code: 1, mobile: 1, profile_picture: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let reservation = await query.selectWithAndOne(dbConstants.dbSchema.reservations, { reservation_id: requestParam.reservation_id }, { _id: 0, customer_id: 1, reservation_id: 1, property_id: 1, status: 1, from_date: 1, to_date: 1, price: 1, parked_date: 1, book_type: 1 });
            // console.log(reservation.customer_id);
            let seeker = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.customer_id }, { _id: 0, customer_id: 1, name: 1, mobile_country_code: 1, mobile: 1, profile_picture: 1 });
            if (!seeker) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            if (!reservation) {
                reject(errors.reservationNotFound(true, requestParam.code));
                return;
            }
            let columnAndValues = {
                property_id: reservation.property_id
            }
            let joinArr = [{
                $lookup: {
                    from: 'parking_spot_types',
                    localField: 'parking_spot_type_id',
                    foreignField: 'parking_spot_type_id',
                    as: 'spotTypesDetails'
                }
            }, {
                $unwind: "$spotTypesDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_terrains',
                    localField: 'parking_spot_terrain_id',
                    foreignField: 'parking_spot_terrain_id',
                    as: 'spotTerrainDetails'
                }
            }, {
                $unwind: "$spotTerrainDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_coverages',
                    localField: 'parking_spot_coverage_id',
                    foreignField: 'parking_spot_coverage_id',
                    as: 'spotCoverageDetails'
                }
            }, {
                $unwind: "$spotCoverageDetails"
            }, {
                $lookup: {
                    from: 'parking_spot_amenities',
                    localField: 'parking_spot_amenity_id',
                    foreignField: 'parking_spot_amenity_id',
                    as: 'spotAmenityDetails'
                }
            }, {
                $lookup: {
                    from: 'access_instructions',
                    localField: 'access_instruction_id',
                    foreignField: 'access_instruction_id',
                    as: 'accessDetails'
                }
            }, {
                $unwind: "$accessDetails"
            }, {
                $lookup: {
                    from: 'customers',
                    localField: 'customer_id',
                    foreignField: 'customer_id',
                    as: 'ownerDetails'
                }
            }, {
                "$unwind": {
                    "path": "$ownerDetails",
                    "preserveNullAndEmptyArrays": true
                }
            }, {
                $lookup: {
                    from: 'property_ratings',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'ratingDetails'
                }
            }, {
                $match: columnAndValues
            }, {
                $sort: { created_at: -1 }
            }, {
                $project: {
                    _id: 0,
                    property_id: 1,
                    status: 1,
                    location: 1,
                    contact_details: 1,
                    property_type: 1,
                    entrance_longitude: 1,
                    latitude: 1,
                    longitude: 1,
                    address: 1,
                    entrance_latitude: 1,
                    entrance_property: 1,
                    customer_id: 1,
                    parking_spot_type_id: 1,
                    parking_spot_type_title: "$spotTypesDetails.title." + requestParam.code,
                    parking_spot_coverage_id: 1,
                    parking_spot_coverage_title: "$spotCoverageDetails.title." + requestParam.code,
                    parking_spot_terrain_id: 1,
                    parking_spot_terrain_title: "$spotTerrainDetails.title." + requestParam.code,
                    parking_spot_amenity_id: 1,
                    parking_spot_amenity_details: "$spotAmenityDetails",
                    access_instruction_id: 1,
                    access_instruction_title: "$accessDetails.title." + requestParam.code,
                    parking_spot_description: 1,
                    parking_spot_photos: 1,
                    from_date_range: 1,
                    to_date_range: 1,
                    available_days: 1,
                    from_reservation_time: 1,
                    to_reservation_time: 1,
                    from_entry_time: 1,
                    to_entry_time: 1,
                    is_auto_activate: 1,
                    distance_from_parking_spot: 1,
                    private_instruction: 1,
                    geo_tag: 1,
                    parking_size: 1,
                    pricing: 1,
                    owner: "$ownerDetails",
                    ratings: "$ratingDetails",
                }
            }];
            let property = await query.joinWithAnd(dbConstants.dbSchema.properties, joinArr);
            property = JSON.parse(JSON.stringify(property))
            await forEach(property, async (elem) => {
                elem.customer_id = reservation.customer_id
                elem.parked_date = timeZone(new Date(reservation.parked_date)).tz(requestParam.time_zone).format('YYYY-MM-DD HH:mm:ss')
                elem.total_price = reservation.price
                elem.from_date = reservation.from_date
                elem.to_date = reservation.to_date
                elem.status = reservation.status
                elem.book_type = reservation.book_type

                if (elem.status == 'reserved') {
                    elem.display_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('lll')
                }
                if (elem.status == 'parked') {
                    let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
                    let to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone);
                    let duration = moment.duration(to_date.diff(todayDate));
                    if (elem.book_type == 'parking_meter') {
                        let from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone);
                        duration = moment.duration(todayDate.diff(from_date));
                    }
                    const hours = parseInt(duration.asHours());
                    const minutes = parseInt(duration.asMinutes()) - hours * 60;
                    if (hours > 0) {
                        elem.display_date = hours + "hrs " + minutes + "mins"
                    }
                    else {
                        elem.display_date = minutes + "mins"
                    }
                }

                elem.from_time = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('lll')
                elem.to_time = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone).format('lll')

                let arr = [];
                _.each(elem.parking_spot_photos, (rec) => {
                    if (rec != '') {
                        arr.push(config.aws.prefix + config.aws.s3.propertyBucket + '/' + rec)
                    }
                });
                _.each(elem.parking_spot_amenity_details, (item) => {
                    item.title = item.title[requestParam.code]
                    item.icon = item.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + item.icon : '';
                    delete item._id
                    delete item.created_at
                    delete item.updated_at
                    delete item.__v
                    delete item.status
                });
                elem.parking_spot_photos = arr

                if (elem.owner) {
                    elem.owner = {
                        customer_id: elem.owner.customer_id,
                        name: elem.owner.name,
                        mobile_country_code: elem.owner.mobile_country_code,
                        mobile: elem.owner.mobile,
                        profile_picture: elem.owner.profile_picture != '' ? config.aws.prefix + config.aws.s3.customerBucket + '/' + elem.owner.profile_picture : ''
                    }
                }
                else {
                    elem.owner = {}
                }

                elem.seeker = {
                    customer_id: seeker.customer_id,
                    name: seeker.name,
                    mobile_country_code: seeker.mobile_country_code,
                    mobile: seeker.mobile,
                    profile_picture: seeker.profile_picture != '' ? config.aws.prefix + config.aws.s3.customerBucket + '/' + seeker.profile_picture : ''
                }

                let rating = 0;
                if (elem.ratings.length > 0) {
                    rating = LD.sumBy(elem.ratings, 'rating')
                    rating = parseFloat(rating) / elem.ratings.length;
                }
                elem.rating = parseFloat(rating).toFixed(1);
                elem.out_of_rating = elem.ratings.length;
                delete elem.ratings

                elem.from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone).format('YYYY-MM-DD HH:mm:ss')
                elem.to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone).format('YYYY-MM-DD HH:mm:ss')

                elem.is_property_rated = false;
                elem.is_owner_rated = false;
                let property_rate = await query.selectWithAndOne(dbConstants.dbSchema.property_ratings, { customer_id: requestParam.customer_id, property_id: elem.property_id }, { _id: 0, property_rating_id: 1 });
                if (property_rate) {
                    elem.is_property_rated = true;
                }
                let owner_rate = await query.selectWithAndOne(dbConstants.dbSchema.seeker_ratings, { seeker_id: requestParam.customer_id, customer_id: elem.customer_id }, { _id: 0, rating_id: 1 });
                if (owner_rate) {
                    elem.is_owner_rated = true;
                }

                // if(elem.status == 'reserved'){
                //     elem.display_date = moment(new Date(elem.from_date)).format('lll')
                // }
                // if(elem.status == 'parked'){
                //     let todayDate = moment(new Date());
                //     let to_date = moment(new Date(elem.to_date))
                //     let duration = moment.duration(to_date.diff(todayDate));
                //     const hours = parseInt(duration.asHours());
                //     const minutes = parseInt(duration.asMinutes()) - hours * 60;
                //     elem.display_date = hours + ":" + minutes
                // }
            });
            property = property[0]

            if (requestParam.latitude && requestParam.longitude) {
                distance.get(
                    {
                        index: 1,
                        origin: '' + requestParam.latitude + ',' + requestParam.longitude + '',
                        destination: '' + property.latitude + ',' + property.longitude + ''
                    },
                    async function (err, data) {
                        if (err) {
                            property.distance_from_parking_spot = property.distance_from_parking_spot
                        }
                        else {
                            property.distance_from_parking_spot = (data.distanceValue / 1000) * 0.6214
                        }
                        resolve(property)
                        return;
                    });
            }
            else {
                resolve(property)
                return;
            }
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : reservation cancel
    Purpose : reservation cancel
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const reservationCancel = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let reservation = await query.selectWithAndOne(dbConstants.dbSchema.reservations, { reservation_id: requestParam.reservation_id }, { _id: 0, reservation_id: 1, property_id: 1, transaction_id: 1, owner_id: 1, is_refund: 1 });
            if (!reservation) {
                reject(errors.reservationNotFound(true, requestParam.code));
                return;
            }

            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
            if (reservation.is_refund == false) {
                let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);
                const refund = await stripe.refunds.create({
                    charge: reservation.transaction_id,
                });
                console.log(refund.id)
            }

            let owner = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.owner_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: 'CANCEL_RESERVATION' }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value[requestParam.code];
                notificationHandler.sendNotification({
                    message: msg,
                    type: 'cancel_reservation',
                    title: config.appName,
                    user_id: reservation.owner_id,
                    device_token: owner ? [owner.device_token] : [],
                    sender_id: reservation.owner_id,
                    reservation_id: reservation.reservation_id,
                    user_type: 'owner',
                    reservation_status: 'cancelled',
                });
                notificationHandler.setLogs({
                    user_id: [reservation.owner_id],
                    user_type: 'owner',
                    message: template.caption_value[requestParam.code],
                    description: msg,
                    type: 'cancel_reservation',
                    address: '',
                    sender_id: reservation.owner_id,
                    reservation_id: reservation.reservation_id,
                    reservation_status: 'cancelled',
                });
            }

            await query.updateSingle(dbConstants.dbSchema.reservations, { status: 'cancelled', is_refund: true }, { reservation_id: requestParam.reservation_id });
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : reservation parked
    Purpose : reservation parked
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const reservationParked = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let reservation = await query.selectWithAndOne(dbConstants.dbSchema.reservations, { reservation_id: requestParam.reservation_id }, { _id: 0, reservation_id: 1, property_id: 1, from_date: 1, status: 1 });
            if (!reservation) {
                reject(errors.reservationNotFound(true, requestParam.code));
                return;
            }
            if (reservation.status == 'cancelled') {
                reject(errors.reservationCancelled(true, requestParam.code));
                return;
            }
            if (reservation.status == 'completed') {
                reject(errors.reservationCompleted(true, requestParam.code));
                return;
            }
            let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
            let from_date = timeZone(new Date(reservation.from_date)).tz(requestParam.time_zone);
            const minutes = from_date.diff(todayDate, 'minutes');
            if (minutes > 5) {
                reject(errors.reservationNotToDoPark(true, requestParam.code));
                return;
            }
            await query.updateSingle(dbConstants.dbSchema.reservations, { status: 'parked', parked_date: new Date() }, { reservation_id: requestParam.reservation_id });
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : extend-parking
    Purpose : extend-parking
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const extendParking = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1, stripe_profile_id: 1, email: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let reservation = await query.selectWithAndOne(dbConstants.dbSchema.reservations, { reservation_id: requestParam.reservation_id }, { _id: 0 });
            if (!reservation) {
                reject(errors.reservationNotFound(true, requestParam.code));
                return;
            }

            let is_free = true;
            let getObj = {}
            getObj['$or'] = [{
                status: 'reserved'
            }, {
                status: 'parked'
            }]
            getObj.property_id = reservation.property_id
            getObj.reservation_id = { $ne: requestParam.reservation_id }
            let reservations = await query.selectWithAnd(dbConstants.dbSchema.reservations, getObj, { _id: 0 });

            let from_date = timeZone(new Date(reservation.from_date)).tz(requestParam.time_zone);
            let to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
            await forEach(reservations, async (rec) => {
                let start_date = timeZone(new Date(rec.from_date)).tz(requestParam.time_zone);
                let last_date = timeZone(new Date(rec.to_date)).tz(requestParam.time_zone);
                //last_date = moment(last_date).add(property.grace_period, 'minutes');
                if (moment(start_date).isBetween(from_date, to_date, null, '[]') || moment(last_date).isBetween(from_date, to_date, null, '[]')) {
                    is_free = false
                }
            });
            if (!is_free) {
                reject(errors.alreadyReservedSlot(true, requestParam.code));
                return;
            }

            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
            let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);

            // let stripeRes = await (new Promise(async (resolve1, reject1) => {
            //     let chargeAmount = Math.round(parseFloat(requestParam.price) * 100).toFixed(2);
            //     stripe.charges.create({
            //         amount: parseFloat(chargeAmount),
            //         currency: "usd",
            //         source: reservation.card_id,
            //         customer: response.stripe_profile_id,
            //         description: response.email,
            //         capture: false
            //     }, async function (err, charge) {
            //         if (err) {
            //             reject(errors.stripeError(true, err.message, requestParam.code));
            //             return;
            //         }
            //         resolve1(charge)
            //     })
            // }));

            /**
             * New Split Payment functionality with stripe
             */
            let ownerDetails = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.owner_id, stripe_connected_account_id: { $ne: "" } }, { _id: 0, bank_details_added: 1, bank_details: 1, stripe_connected_account_id: 1 });
            let chargeAmount = parseFloat(requestParam.price);
            let { error, stripeRes, company_earning, owner_earning } = await createReservationChargesStripe({
                settings,
                is_payment_confirmed: false,
                amount: chargeAmount,
                card_id: reservation.card_id,
                customer_stripe_id: response.stripe_profile_id,
                owner_connected_account: ownerDetails.stripe_connected_account_id
            })
            if (error) {
                // error code here
                console.log(error)
                reject(error)
                return;
            }
            requestParam.extend_transaction_id = stripeRes.id

            requestParam.to_date = new Date(requestParam.to_date);
            let price = reservation.price + parseFloat(requestParam.price)

            let newPrice = Number(requestParam.price);
            newPrice = (newPrice * 0.029) + 0.30;
            newPrice = Number(requestParam.price) - newPrice;

            let company_earn = reservation.company_earn + company_earning;
            let owner_earn = reservation.owner_earn + owner_earning

            // let company_earn = ((parseFloat(newPrice)) * (reservation.company_percentage)) / 100
            // let owner_earn = parseFloat(newPrice) - parseFloat(company_earn)

            await query.updateSingle(dbConstants.dbSchema.reservations, { extend_transaction_id: requestParam.extend_transaction_id, to_date: requestParam.to_date, price: price, company_earn: company_earn, owner_earn: owner_earn }, { reservation_id: requestParam.reservation_id });
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : property-rating
    Purpose : property-rating
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const propertyRating = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: requestParam.property_id }, { _id: 0, property_id: 1 });
            if (!property) {
                reject(errors.propertyNotFound(true, requestParam.code));
                return;
            }
            requestParam.rating = parseFloat(requestParam.rating)
            await query.insertSingle(dbConstants.dbSchema.property_ratings, requestParam);
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : owner-rating
    Purpose : owner-rating
    Original Author : Gaurav Patel
    Created At : 03rd May 2021
*/
const ownerRating = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let owner = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.owner_id }, { _id: 0, customer_id: 1 });
            if (!owner) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            await query.insertSingle(dbConstants.dbSchema.owner_ratings, requestParam);
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

/*
    Name : complete-reservation
    Purpose : complete-reservation
    Original Author : Gaurav Patel
    Created At : 25th May 2021
*/
// const completeReservation = async(requestParam) => {
//     return new Promise(async(resolve, reject) => {
//         try {
//             sendNotification10MinsBefore(requestParam);
//             let columnAndValue = {};
//             columnAndValue = {
//                 $or: [{
//                     status: 'reserved'
//                 }, {
//                     status: 'parked'
//                 }]
//             };
//             let data = await query.selectWithAnd(dbConstants.dbSchema.reservations, columnAndValue, { _id: 0} );
//             await forEach(data, async(elem) => {
//                 let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
//                 let to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone);
//                 const minutes = to_date.diff(todayDate, 'minutes');
//                 if(minutes == 10){
//                     seekerNotifyBefore10minsComplete(elem);
//                 }
//                 if(minutes <= -10 && elem.book_type == 'normal'){
//                     let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0, is_payment_live: 1, company_percentage:1} );
//                     let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);

//                     let stripeRes = await (new Promise(async(resolve1, reject1) => {
//                         stripe.charges.capture(elem.transaction_id, (err, charge) => {
//                             if(err){
//                                 resolve1('')
//                             }
//                             resolve1(charge ? charge.id : '')
//                         });
//                     }));

//                     let obj = {status:'completed', transaction_id:stripeRes, completed_date: new Date()};

//                     if(elem.extend_transaction_id !=''){
//                         stripeRes = await (new Promise(async(resolve1, reject1) => {
//                             stripe.charges.capture(elem.extend_transaction_id, (err, charge) => {
//                                 if(err){
//                                     resolve1('')
//                                     return;
//                                 }
//                                 resolve1(charge ? charge.id : '')
//                             });
//                         }));
//                         obj.extend_transaction_id = stripeRes
//                     }
//                     await query.updateSingle(dbConstants.dbSchema.reservations, obj, { reservation_id: elem.reservation_id });
//                 }
//             });
//             resolve({})
//             return;
//         } catch (error) {
//             console.log(error)
//             reject(error)
//             return
//         }
//     })
// };

/*
    Name : complete-reservation
    Purpose : complete-reservation
    Original Author : Gaurav Patel
    Created At : 25th May 2021
*/
const cronBeforeAfterSendPush = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            sendNotification10MinsBefore(requestParam);
            let columnAndValue = {};
            columnAndValue = {
                $or: [{
                    status: 'reserved'
                }, {
                    status: 'parked'
                }]
            };
            let data = await query.selectWithAnd(dbConstants.dbSchema.reservations, columnAndValue, { _id: 0 });
            await forEach(data, async (elem) => {
                let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
                let to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone);
                const minutes = to_date.diff(todayDate, 'minutes');
                if (minutes == 10) {
                    elem.code = '10_MINS_BEFORE_COMPLETE_RES_NOTIFY_TO_SEEKER'
                    elem.push_type = '10_min_before_complete_reservation'
                    seekerNotifyBefore10minsComplete(elem);
                }
                if (minutes == 15) {
                    elem.code = '15_MINS_BEFORE_COMPLETE_RES_NOTIFY_TO_SEEKER'
                    elem.push_type = '15_min_before_complete_reservation'
                    seekerNotifyBefore10minsComplete(elem);
                }

                let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: requestParam.property_id }, { _id: 0, property_id: 1, customer_id: 1, address: 1, grace_period: 1 });
                let minutesAfterComplete = todayDate.diff(to_date, 'minutes');
                if (minutesAfterComplete == 10) {
                    elem.code = 'NOTIFY_TO_SEEKER_AFTER_COMPLETE_PARKING_TIME'
                    elem.push_type = 'parking_time_expried_without_grace_period'
                    seekerNotifyBefore10minsComplete(elem);
                }
                if (property) {
                    let grace_period = property.grace_period ? property.grace_period : 10;
                    grace_period = parseFloat(grace_period) + 10;

                    if (minutesAfterComplete == grace_period) {
                        elem.code = 'NOTIFY_TO_SEEKER_AFTER_COMPLETE_PARKING_TIME_WITH_GRACE_PERIOD'
                        elem.push_type = 'parking_time_expried_with_grace_period'
                        seekerNotifyBefore10minsComplete(elem);
                    }
                }
            });
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const sendNotification10MinsBefore = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let data = await query.selectWithAnd(dbConstants.dbSchema.reservations, { status: 'reserved' }, { _id: 0, reservation_id: 1, from_date: 1, to_date: 1, customer_id: 1, property_id: 1, status: 1 });
            await Promise.all(data.map(async (elem) => {
                let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
                let from_date = timeZone(new Date(elem.from_date)).tz(requestParam.time_zone);
                const minutes = from_date.diff(todayDate, 'minutes');
                if (minutes == 10) {
                    seekerNotifyBefore10mins(elem);
                }
                if (minutes == 5) {
                    let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: elem.customer_id }, { name: 1, mobile_country_code: 1, mobile: 1 });
                    let template = await query.selectWithAndOne(dbConstants.dbSchema.sms_templates, { code: 'SEEKER_MSG_BEFORE_5_MINS' }, { _id: 0, value: 1 });
                    if (template && response) {
                        let msg = template.value['EN']
                        msg = msg.replace('#NAME#', response.name)
                        sendSMS({ message: msg, mobile_country_code: response.mobile_country_code, mobile: response.mobile });
                    }
                }
            }));
            return false;
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

const seekerNotifyBefore10mins = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: requestParam.property_id }, { _id: 0, address: 1 });
            let customer = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: '10_MINS_BEFORE_NOTIFY_TO_SEEKER' }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value['EN'];
                notificationHandler.sendNotification({
                    message: msg,
                    type: '10_min_before_reservation',
                    title: config.appName,
                    user_id: requestParam.customer_id,
                    device_token: customer ? [customer.device_token] : [],
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: 'seeker',
                    reservation_status: requestParam.status,
                });
                notificationHandler.setLogs({
                    user_id: [requestParam.customer_id],
                    user_type: 'seeker',
                    message: template.caption_value['EN'],
                    description: msg,
                    type: '10_min_before_reservation',
                    address: property ? property.address : '',
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    reservation_status: requestParam.status,
                });
            }
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

const seekerNotifyBefore10minsComplete = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: requestParam.property_id }, { _id: 0, address: 1 });
            let customer = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: requestParam.code }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value['EN'];
                notificationHandler.sendNotification({
                    message: msg,
                    type: requestParam.push_type,
                    title: config.appName,
                    user_id: requestParam.customer_id,
                    device_token: customer ? [customer.device_token] : [],
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    user_type: 'seeker',
                    reservation_status: requestParam.status,
                });
                notificationHandler.setLogs({
                    user_id: [requestParam.customer_id],
                    user_type: 'seeker',
                    message: template.caption_value['EN'],
                    description: msg,
                    type: requestParam.push_type,
                    address: property ? property.address : '',
                    sender_id: requestParam.customer_id,
                    reservation_id: requestParam.reservation_id,
                    reservation_status: requestParam.status,
                });
            }
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};


const completeReservationWithApp = async (requestParam, done) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1, stripe_profile_id: 1, email: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let reservation = await query.selectWithAndOne(dbConstants.dbSchema.reservations, { reservation_id: requestParam.reservation_id }, { _id: 0 });
            if (!reservation) {
                reject(errors.reservationNotFound(true, requestParam.code));
                return;
            }

            if (reservation.status != 'parked') {
                reject(errors.reservationNotParked(true, requestParam.code));
                return;
            }
            // requestParam.to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
            // if(requestParam.time_zone == 'Europe/Kiev'){
            //     requestParam.to_date = moment(requestParam.to_date).add(150, 'minutes');
            // }

            // if(requestParam.time_zone == 'America/Los_Angeles'){
            //     requestParam.to_date = moment(requestParam.to_date).add(750, 'minutes');
            // }

            congratsSendNotificationAfterCompleteReservation(reservation, requestParam);

            if (reservation.book_type == 'parking_meter') {
                resolve(parkingMeterCompleteReservation(requestParam, reservation, response));
                return;
            }
            let updateObj = {
                to_date: new Date(requestParam.to_date),
                status: 'completed',
                completed_date: new Date()
            }

            let final_price = reservation.price;
            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
            let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);

            let to_date = timeZone(new Date(reservation.to_date)).tz(requestParam.time_zone);
            let final_to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
            let duration = moment.duration(final_to_date.diff(to_date));
            const mins = parseInt(duration.asMinutes());
            console.log("Final price", final_price)
            console.log("Final mins", mins)

            let new_company_earning = 0;
            let new_owner_earning = 0;
            if (mins > 0) {
                let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: reservation.property_id }, { _id: 0, property_id: 1, pricing: 1 });
                if (!property) {
                    reject(errors.propertyNotFound(true, requestParam.code));
                    return;
                }

                // let stripeRes = await (new Promise(async (resolve1, reject1) => {
                //     let price = property.pricing.hourly_overage_rate * (mins / 60);
                //     if (parseFloat(price) < 1) {
                //         price = 1
                //     }
                //     updateObj.overtime_charge = price;
                //     final_price += price
                //     let chargeAmount = Math.round(parseFloat(price) * 100).toFixed(2);
                //     stripe.charges.create({
                //         amount: parseFloat(chargeAmount),
                //         currency: "usd",
                //         source: reservation.card_id,
                //         customer: response.stripe_profile_id,
                //         description: response.email,
                //     }, async function (err, charge) {
                //         console.log(err)
                //         if (err) {
                //             reject(errors.stripeError(true, err.message, requestParam.code));
                //             return;
                //         }
                //         resolve1(charge)
                //     })
                // }));

                let ownerDetails = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.customer_id, stripe_connected_account_id: { $ne: "" } }, { _id: 0, bank_details_added: 1, bank_details: 1, stripe_connected_account_id: 1 });
                let price = property.pricing.hourly_overage_rate * (mins / 60);
                if (parseFloat(price) < 1) {
                    price = 1
                }
                updateObj.overtime_charge = price;
                final_price += price
                let chargeAmount = parseFloat(price);
                let { error, stripeRes, company_earning, owner_earning } = await createReservationChargesStripe({
                    settings,
                    is_payment_confirmed: true,
                    amount: chargeAmount,
                    card_id: reservation.card_id,
                    customer_stripe_id: response.stripe_profile_id,
                    owner_connected_account: ownerDetails.stripe_connected_account_id
                })
                if (error) {
                    // error code here
                    console.log(error)
                    reject(error)
                    return;
                }
                new_company_earning += company_earning;
                new_owner_earning += owner_earning;
                updateObj.over_charge_transaction_id = stripeRes.id
            }

            // let stripeRes1 = await (new Promise(async (resolve1, reject1) => {
            //     stripe.charges.capture(reservation.transaction_id, (err, charge) => {
            //         console.log("stripeRes1")
            //         console.log(err)
            //         if (err) {
            //             reject(errors.stripeError(true, err.message, requestParam.code));
            //             return;
            //         }
            //         resolve1(charge)
            //     });
            // }));

            /**
            * New payment capture code
            */
            let stripeRes1 = await stripePaymentConfirmed(settings, reservation.transaction_id);
            if (stripeRes1.error) {
                console.log(stripeRes1.error)
                reject(errors.stripeError(true, stripeRes1.error, requestParam.code));
                return;
            }

            if (reservation.extend_transaction_id != '') {
                // let stripeRes2 = await (new Promise(async (resolve1, reject1) => {
                //     stripe.charges.capture(reservation.extend_transaction_id, (err, charge) => {
                //         console.log("stripeRes2")
                //         console.log(err)
                //         if (err) {
                //             reject(errors.stripeError(true, err.message, requestParam.code));
                //             return;
                //         }
                //         resolve1(charge)
                //     });
                // }));

                /**
                 * New payment capture code
                 */
                let stripeRes2 = await stripePaymentConfirmed(settings, reservation.extend_transaction_id);
                if (stripeRes2.error) {
                    console.log(stripeRes2.error)
                    reject(errors.stripeError(true, stripeRes2.error, requestParam.code));
                    return;
                }
                updateObj.extend_transaction_id = stripeRes2.id
            }
            let newPrice = Number(final_price);
            newPrice = (newPrice * 0.029) + 0.30;
            newPrice = Number(final_price) - newPrice;
            // requestParam.transaction_id = stripeRes.id
            // let company_earn = ((parseFloat(newPrice)) * (reservation.company_percentage)) / 100
            // let owner_earn = parseFloat(newPrice) - parseFloat(company_earn)

            let company_earn = new_company_earning + reservation.company_earn
            let owner_earn = new_owner_earning + reservation.owner_earn

            updateObj.price = final_price;
            updateObj.company_earn = company_earn;
            updateObj.owner_earn = owner_earn;
            updateObj.transaction_id = stripeRes1.id;

            await query.updateSingle(dbConstants.dbSchema.reservations, updateObj, { reservation_id: requestParam.reservation_id });
            console.log("Done");
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const congratsSendNotificationAfterCompleteReservation = async (reservation, requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            // NOTIFICATION TO OWNER

            let owner = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.owner_id }, { _id: 0, device_token: 1 });
            let template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: 'CONGRATS_MSG_TO_OWNER' }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg = template.value[requestParam.code];
                notificationHandler.sendNotification({
                    message: msg,
                    type: 'congrats_owner_complete_reservation',
                    title: config.appName,
                    user_id: reservation.owner_id,
                    device_token: owner ? [owner.device_token] : [],
                    sender_id: reservation.owner_id,
                    reservation_id: reservation.reservation_id,
                    user_type: 'owner',
                    reservation_status: 'completed',
                });
                notificationHandler.setLogs({
                    user_id: [reservation.owner_id],
                    user_type: 'owner',
                    message: template.caption_value[requestParam.code],
                    description: msg,
                    type: 'congrats_owner_complete_reservation',
                    address: '',
                    sender_id: reservation.owner_id,
                    reservation_id: reservation.reservation_id,
                    reservation_status: 'completed',
                });
            }

            // END NOTIFICATION TO OWNER

            // NOTIFICATION TO SEEKER

            let seeker = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: reservation.customer_id }, { _id: 0, device_token: 1 });
            template = await query.selectWithAndOne(dbConstants.dbSchema.push_templates, { code: 'CONGRATS_MSG_TO_SEEKER' }, { _id: 0, value: 1, caption_value: 1 }, { created_at: 1 });
            if (template) {
                let msg1 = template.value[requestParam.code];
                notificationHandler.sendNotification({
                    message: msg1,
                    type: 'congrats_seeker_complete_reservation',
                    title: config.appName,
                    user_id: reservation.customer_id,
                    device_token: seeker ? [seeker.device_token] : [],
                    sender_id: reservation.customer_id,
                    reservation_id: reservation.reservation_id,
                    user_type: 'seeker',
                    reservation_status: 'completed',
                });
                notificationHandler.setLogs({
                    user_id: [reservation.customer_id],
                    user_type: 'seeker',
                    message: template.caption_value[requestParam.code],
                    description: msg1,
                    type: 'congrats_seeker_complete_reservation',
                    address: '',
                    sender_id: reservation.customer_id,
                    reservation_id: reservation.reservation_id,
                    reservation_status: 'completed',
                });
            }

            // END NOTIFICATION TO SEEKER
            return false;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const parkingMeterCompleteReservation = async (requestParam, reservation, response) => {
    return new Promise(async (resolve, reject) => {
        try {
            let updateObj = {
                to_date: new Date(requestParam.to_date),
                status: 'completed',
                completed_date: new Date()
            }
            let final_price = 0;
            let settings = await query.selectWithAndOne(dbConstants.dbSchema.settings, {}, { _id: 0 });
            let stripe = require("stripe")(settings.is_payment_live ? config.stripeInfo.live_key : config.stripeInfo.test_key);

            let from_date = timeZone(new Date(reservation.from_date)).tz(requestParam.time_zone);
            let final_to_date = timeZone(new Date(requestParam.to_date)).tz(requestParam.time_zone);
            let duration = moment.duration(final_to_date.diff(from_date));
            const mins = parseInt(duration.asMinutes());
            if (mins > 0) {
                let property = await query.selectWithAndOne(dbConstants.dbSchema.properties, { property_id: reservation.property_id }, { _id: 0, property_id: 1, pricing: 1 });
                if (!property) {
                    reject(errors.propertyNotFound(true, requestParam.code));
                    return;
                }
                let stripeRes = await (new Promise(async (resolve1, reject1) => {
                    let price = property.pricing.hourly_rate * (mins / 60);
                    if (parseFloat(price) < 1) {
                        price = 1
                    }
                    final_price += price
                    let chargeAmount = Math.round(parseFloat(price) * 100).toFixed(2);
                    stripe.charges.create({
                        amount: parseInt(chargeAmount),
                        currency: "usd",
                        source: reservation.card_id,
                        customer: response.stripe_profile_id,
                        description: response.email,
                    }, async function (err, charge) {
                        if (err) {
                            reject(errors.stripeError(true, err.message, requestParam.code));
                            return;
                        }
                        resolve1(charge)
                    })
                }));
                updateObj.transaction_id = stripeRes.id
            }

            let newPrice = Number(final_price);
            newPrice = (newPrice * 0.029) + 0.30;
            newPrice = Number(final_price) - newPrice;

            let company_earn = ((parseFloat(newPrice)) * (reservation.company_percentage)) / 100
            let owner_earn = parseFloat(newPrice) - parseFloat(company_earn)

            updateObj.price = final_price;
            updateObj.company_earn = company_earn;
            updateObj.owner_earn = owner_earn;

            await query.updateSingle(dbConstants.dbSchema.reservations, updateObj, { reservation_id: requestParam.reservation_id });
            resolve({})
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const getReservedParkedList = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let response = await query.selectWithAndOne(dbConstants.dbSchema.customers, { customer_id: requestParam.customer_id }, { _id: 0, customer_id: 1 });
            if (!response) {
                reject(errors.userNotFound(true, requestParam.code));
                return;
            }
            let columnAndValue = {};
            columnAndValue = {
                $or: [{
                    status: 'parked'
                }, {
                    status: 'reserved'
                }]
            };
            columnAndValue.customer_id = requestParam.customer_id;
            let joinArr = [{
                $lookup: {
                    from: 'properties',
                    localField: 'property_id',
                    foreignField: 'property_id',
                    as: 'propDetails'
                }
            }, {
                $unwind: "$propDetails"
            }, {
                $match: columnAndValue
            }, {
                $sort: { created_at: -1 }
            }, {
                $project: {
                    _id: 0,
                    reservation_id: "$reservation_id",
                    property_id: "$property_id",
                    geo_tag: "$propDetails.geo_tag",
                    status: "$status",
                    private_instruction: "$propDetails.private_instruction",
                }
            }];
            let reservations = await query.joinWithAnd(dbConstants.dbSchema.reservations, joinArr);
            resolve(reservations)
            return;
        } catch (error) {
            console.log(error)
            reject(error)
            return
        }
    })
};

const cancelReservationFromCron = async (requestParam) => {
    return new Promise(async (resolve, reject) => {
        try {
            let data = await query.selectWithAnd(dbConstants.dbSchema.reservations, { status: 'reserved' }, { _id: 0, reservation_id: 1, from_date: 1, to_date: 1, customer_id: 1, property_id: 1, status: 1 });
            async.forEachSeries(data, async function (elem, Callback_s1) {
                let todayDate = timeZone(new Date()).tz(requestParam.time_zone);
                let to_date = timeZone(new Date(elem.to_date)).tz(requestParam.time_zone);
                const minutes = to_date.diff(todayDate, 'minutes');
                if (minutes < 0) {
                    let val = await reservationCancel({ code: 'EN', customer_id: elem.customer_id, reservation_id: elem.reservation_id });
                }
                Callback_s1();
            }, function () {
                resolve({});
                return;
            });
        } catch (error) {
            console.log(error)
            return false;
        }
    })
};

const nearParkingList = async(requestParam)=>{
    return new Promise(async (resolve, reject) => {
        try {
            console.log("latitude",requestParam)

            let joinArr = [{
                $geoNear: {
                    includeLocs: "location",
                    distanceField: "distance",
                    near: { type: 'Point', coordinates: [parseFloat(requestParam.longitude), parseFloat(requestParam.latitude)] },
                    maxDistance: 10000,
                    spherical: true
                } 
            }];
            let property = await query.joinWithAnd(dbConstants.dbSchema.properties, joinArr);

            resolve(property)
            return;
            
        } catch (error) {
            console.log(error)
            return false;
        }
    })

};


module.exports = {
    parkingSpotList,
    favourite,
    favouriteList,
    reservation,
    reservationList,
    reservationDetails,
    reservationCancel,
    reservationParked,
    extendParking,
    propertyRating,
    ownerRating,
    //completeReservation,
    sendNotification10MinsBefore,
    seekerNotifyBefore10minsComplete,
    completeReservationWithApp,
    parkingMeterCompleteReservation,
    getReservedParkedList,
    cronBeforeAfterSendPush,
    cancelReservationFromCron,
    nearParkingList
};