'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const Currency = require('./../../models/currency');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.currency_id){
		columnAndValue.currency_id = req.query.currency_id
	}
    if(req.query.status){
        columnAndValue.status = req.query.status
    }
	query.selectWithAndFilter(dbConstants.dbSchema.currencies, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, response) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.currencies);
            done(errors.internalServer(true), null);
            return;
        }
        done(null,response)
    });
};

const create = function(requestParam,done){
	query.insertSingle(dbConstants.dbSchema.currencies,requestParam,function (error, response) {
		if (error) {
			logger('Error: can not create');
			done(error, null);
			return;
		}
		done(null, response);
	});
};

const update = function(requestParam,done){
	query.updateSingle(dbConstants.dbSchema.currencies,requestParam, { 'currency_id':requestParam.currency_id},function (error, response) {
		if (error) {
			logger('Error: can not update');
			done(error, null);
			return;
		}
		done(null, response);
	});
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.currencies, {
            'currency_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });        
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.currencies, columnsToUpdate, {
            'currency_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update ');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update,
};