'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const Measurement = require('./../../models/measurement');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.measurement_id){
		columnAndValue.measurement_id = req.query.measurement_id
	}
    query.selectWithAndFilter(dbConstants.dbSchema.measurements, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, response) => {
        if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.measurements);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.measurement_id){
            response = response[0]
            response = JSON.parse(JSON.stringify(response));
            query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'}, {}, function(error, languages) {
                _.each(languages, (element, index, list) => {
                    response[element.code] = response.title[element.code];
                });
                done(null,response)
            });
        }
        else{
            _.each(response, (elem) => {
                elem.title = elem.title.EN
            })
            done(null, response)
        }
    });
};

const create = function(requestParam,done){
    let exists = false;
    let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        async.forEachSeries(languages, async function(element, callbackSingleRec) {
            query.selectWithAndOne(dbConstants.dbSchema.measurements, {'title.EN': requestParam[element.code]}, {
                measurement_id: 1
            }, async(error, duplicate) => {
                if(duplicate){
                    exists = true
                }
                title[element.code] = requestParam[element.code];
                callbackSingleRec();
            });
        }, function(){
            if (exists) {
                done(errors.duplicateUser(true), null);
                return;
            }
            requestParam.title = title;
        	query.insertSingle(dbConstants.dbSchema.measurements,requestParam,function (error, response) {
        		if (error) {
        			logger('Error: can not create mode');
        			done(error, null);
        			return;
        		}
        		done(null, response);
        	});
        });
    });
};


const update = function(requestParam,done){
    let exists = false;
    let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        async.forEachSeries(languages, async function(element, callbackSingleRec) {
            query.selectWithAndOne(dbConstants.dbSchema.measurements, {'title.EN': requestParam[element.code], measurement_id:{$ne: requestParam.measurement_id}}, {
                measurement_id: 1
            }, async(error, duplicate) => {
                if(duplicate){
                    exists = true
                }
                title[element.code] = requestParam[element.code];
                callbackSingleRec();
            });
        }, function(){
            if (exists) {
                done(errors.duplicateUser(true), null);
                return;
            }
            requestParam.title = title;
            query.updateSingle(dbConstants.dbSchema.measurements,requestParam, { 'measurement_id':requestParam.measurement_id},function (error, response) {
                if (error) {
                    logger('Error: can not update mode');
                    done(error, null);
                    return;
                }
                done(null, response);
            });
        });
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.measurements, {
            'measurement_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });        
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.measurements, columnsToUpdate, {
            'measurement_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update ');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update
};