'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let asyncLoop = require('async');
let _ = require('underscore');
const modules = require('./../../models/module');

const get = function(req, done) {
    let columnAndValue = {}
    if (req.query.module_id) {
        columnAndValue.module_id = req.query.module_id
    }
    if (req.query.status) {
        columnAndValue.status = req.query.status
    }
    query.selectWithAndFilter(dbConstants.dbSchema.modules, columnAndValue, {
        _id: 0,
    }, {
        order_no: 1
    }, {}, (error, modules) => {
        if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.modules);
            done(errors.internalServer(true), null);
            return;
        }
        done(null, modules)
    });
};

const create = function(requestParam, done) {
    query.insertSingle(dbConstants.dbSchema.modules, requestParam, function(error, role) {
        if (error) {
            logger('Error: can not create role');
            done(error, null);
            return;
        }
        insertAccessRight(requestParam)
        done(null, role);
    });
};

const update = function(requestParam, done) {
    query.updateSingle(dbConstants.dbSchema.modules, requestParam, {
        'module_id': requestParam.module_id
    }, function(error, role) {
        if (error) {
            logger('Error: can not update role');
            done(error, null);
            return;
        }
        done(null, role);
    });
};

const action = (requestParam, done) => {
    if (requestParam['type'] == "delete") {
        deleteAccessRight(requestParam)
        query.removeMultiple(dbConstants.dbSchema.modules, {
            'module_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            done(null, data);
        });
    } else {
        let columnsToUpdate = {
            status: requestParam['type']
        };
        query.updateMultiple(dbConstants.dbSchema.modules, columnsToUpdate, {
            'module_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update ');
                done(error, null);
                return;
            }
            done(null, data);
        });
    }
};

const insertAccessRight = (requestParam, done) => {
    query.selectWithAndFilter(dbConstants.dbSchema.access_rights, {}, {
        _id: 0,
        role_id: 1,
        module: 1,
        access_right_id: 1,
    }, {
        created_at: 1
    }, {}, (error, response) => {
        response = JSON.parse(JSON.stringify(response))
        let obj = {
            is_view: true,
            is_add: true,
            is_edit: true,
            is_delete: true,
        };
        asyncLoop.forEachSeries(response, async function(elem, Callback_s1) {
            elem.module[requestParam.title] = obj;
            query.updateSingle(dbConstants.dbSchema.access_rights, elem, {
                'access_right_id': elem.access_right_id
            }, function(error, role) {
                Callback_s1();
            });
        }, function() {
            return false;
        });
    });
};

const deleteAccessRight = (requestParam, done) => {
    query.selectWithAndFilter(dbConstants.dbSchema.modules, {
        module_id: {
            $in: requestParam['ids']
        }
    }, {
        _id: 0,
        module_id: 1,
        title: 1,
    }, {}, {}, (error, modules) => {
        query.selectWithAndFilter(dbConstants.dbSchema.access_rights, {}, {
            _id: 0,
            role_id: 1,
            module: 1,
            access_right_id: 1,
        }, {
            created_at: 1
        }, {}, (error, response) => {
            asyncLoop.forEachSeries(modules, async function(module, Callback_s2) {
                asyncLoop.forEachSeries(response, async function(elem, Callback_s1) {
                    delete elem.module[module.title];
                    query.updateSingle(dbConstants.dbSchema.access_rights, elem, {
                        'access_right_id': elem.access_right_id
                    }, function(error, role) {
                        Callback_s1();
                    });
                }, function() {
                    Callback_s2();
                });
            }, function() {
                return false;
            });
        });
    });
};


module.exports = {
    get,
    create,
    action,
    update,
    insertAccessRight
};