'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
const config = require('./../../config');
let async = require('async');
let _ = require('underscore');
const Parking_spot_amenity = require('./../../models/parking-spot-amenity');
const commonHandler = require('./../../model_handlers/backend/common-handler');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.parking_spot_amenity_id){
		columnAndValue.parking_spot_amenity_id = req.query.parking_spot_amenity_id
	}
	query.selectWithAndFilter(dbConstants.dbSchema.parking_spot_amenities, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, spot_type) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.parking_spot_amenities);
            done(errors.internalServer(true), null);
            return;
        }
        if(req.query.parking_spot_amenity_id){
            spot_type = spot_type[0]
            spot_type = JSON.parse(JSON.stringify(spot_type));
            spot_type.icon = spot_type.icon != '' ? config.aws.prefix + config.aws.s3.amenityBucket + '/' + spot_type.icon : '';
            query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'}, {}, function(error, languages) {
                _.each(languages, (element, index, list) => {
                    spot_type[element.code] = spot_type.title[element.code];
                });
                done(null,spot_type)
            });
        }
        else{
            done(null,spot_type)
        }
    });
};

const create = function(requestParam, req, done){
	let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, async function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
        requestParam.icon = await new Promise((resolve, reject) => {
            commonHandler.uploadImage(req.files.icon, config.aws.s3.amenityBucket, (error, path) => {
                resolve(path)
            });
        });
        query.insertSingle(dbConstants.dbSchema.parking_spot_amenities,requestParam,function (error, spot_type) {
            if (error) {
                logger('Error: can not create spot_type');
                done(error, null);
                return;
            }
            done(null, spot_type);
        });
    });
};

const update = function(requestParam, req, done){
    let title = new Object();
    query.selectWithAnd(dbConstants.dbSchema.languages,{status:'active'},{}, function (error, languages) {
        _.each(languages, (element, index, list) => {
            title[element.code] = requestParam[element.code];
        });
        requestParam.title = title;
        if (requestParam.change_logo) {
            query.selectWithAndOne(dbConstants.dbSchema.parking_spot_amenities, {
                parking_spot_amenity_id: requestParam.parking_spot_amenity_id
            }, {
                parking_spot_amenity_id: 1,
                icon: 1,
            }, async(error, user) => {
                let fileObjects = [];
                if(requestParam.change_logo){
                    fileObjects.push({
                        Key: 'parksie/amenities/' + /[^/]*$/.exec(user.icon)[0]
                    })
                }
                commonHandler.removeMultipleImages(fileObjects, async(error, path) => {
                    requestParam.icon = await new Promise((resolve, reject) => {
                        commonHandler.uploadImage(req.files.icon, config.aws.s3.amenityBucket, (error, path) => {
                            resolve(path)
                        });
                    });
                    query.updateSingle(dbConstants.dbSchema.parking_spot_amenities,requestParam, { 'parking_spot_amenity_id':requestParam.parking_spot_amenity_id},function (error, spot_type) {
                        if (error) {
                            logger('Error: can not update spot_type');
                            done(error, null);
                            return;
                        }
                        done(null, spot_type);
                    });
                });
            });
        }
        else{
            delete requestParam.icon
        	query.updateSingle(dbConstants.dbSchema.parking_spot_amenities,requestParam, { 'parking_spot_amenity_id':requestParam.parking_spot_amenity_id},function (error, spot_type) {
        		if (error) {
        			logger('Error: can not update spot_type');
        			done(error, null);
        			return;
        		}
        		done(null, spot_type);
        	});
        }
    });
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.selectWithAnd(dbConstants.dbSchema.parking_spot_amenities, {
            parking_spot_amenity_id: {
                $in: requestParam.ids
            }
        }, {
            _id: 0,
            parking_spot_amenity_id: 1,
            icon: 1,
        }, (error, user) => {
            let fileObjects = [];
            for (var i = 0; i < user.length; i++) {
                fileObjects.push({
                    Key: 'parksie/amenities/' + /[^/]*$/.exec(user[i].icon)[0]
                })
            }
            commonHandler.removeMultipleImages(fileObjects, (error, path) => {
                query.removeMultiple(dbConstants.dbSchema.parking_spot_amenities, {
                    'parking_spot_amenity_id': {
                        $in: requestParam['ids']
                    }
                }, function(error, data) {
                    if (error) {
                        logger('Error: can not delete ');
                        done(error, null);
                        return;
                    }
                    done(null, data);
                });
            });
        });         
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.parking_spot_amenities, columnsToUpdate, {
            'parking_spot_amenity_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update spot_type');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};


module.exports = {
	get,
	create,
	action,
	update,
};