'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
const queryApi = require('./../../utils/query-creator-api');
let async = require('async');
let _ = require('underscore');
const LD = require('lodash');
const monthName = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
const moment = require('moment');
const { forEach } = require('p-iteration');

const totalSales = async(req, done) => {
    let year = new Date().getFullYear()
    if(req.query.year){
        year = parseFloat(req.query.year)
    }
    let columnAndValues = {};
    let promise = [];
    for (let x in monthName) {
        promise.push(await overallEarningSalesCountByMonth(monthName[x], x, columnAndValues, 'price', year, 'total_sale'))
    }
    Promise.all(promise)
    .then(async result => {
        done(null, result)
    });
};

const expectedSales = async(req, done) => {
    let year = new Date().getFullYear()
    if(req.query.year){
        year = parseFloat(req.query.year)
    }
    let columnAndValues = {};
    let promise = [];
    for (let x in monthName) {
        promise.push(await overallEarningSalesCountByMonth(monthName[x], x, columnAndValues, 'price', year, 'expected_sale'))
    }
    Promise.all(promise)
    .then(async result => {
        done(null, result)
    });
};

const totalEarnings = async(req, done) => {
    let year = new Date().getFullYear()
    if(req.query.year){
        year = parseFloat(req.query.year)
    }
    let columnAndValues = {};
    let promise = [];
    for (let x in monthName) {
        promise.push(await overallEarningSalesCountByMonth(monthName[x], x, columnAndValues, 'company_earn', year, 'total_earning'))
    }
    Promise.all(promise)
    .then(async result => {
        done(null, result)
    });
};

const expectedEarnings = async(req, done) => {
    let year = new Date().getFullYear()
    if(req.query.year){
        year = parseFloat(req.query.year)
    }
    let columnAndValues = {};
    let promise = [];
    for (let x in monthName) {
        promise.push(await overallEarningSalesCountByMonth(monthName[x], x, columnAndValues, 'company_earn', year, 'expected_earning'))
    }
    Promise.all(promise)
    .then(async result => {
        done(null, result)
    });
};

const totalPayouts = async(req, done) => {
    let year = new Date().getFullYear()
    if(req.query.year){
        year = parseFloat(req.query.year)
    }
    let columnAndValues = {};
    let promise = [];
    for (let x in monthName) {
        promise.push(await overallEarningSalesCountByMonth(monthName[x], x, columnAndValues, 'owner_earn', year, 'total_payout'))
    }
    Promise.all(promise)
    .then(async result => {
        done(null, result)
    });
};

const overallEarningSalesCountByMonth = (month, index, columnAndValues, column, year, type) => {
    return new Promise(async(resolve, reject) => {
        var date = new Date(),
            y = year,
            m = parseInt(index);
        var firstDay = new Date(y, m, 1);
        firstDay.setHours(0, 0, 0, 0);
        var lastDay = new Date(y, m + 1, 0);
        lastDay.setHours(23, 59, 59, 999);
        let compairData = {
            status: 'completed',
            completed_date: {
                $gte: firstDay,
                $lt: lastDay
            }
        }
        if(type == 'expected_sale' || type == 'expected_earning'){
            delete compairData.status
        }
        query.selectWithAnd(dbConstants.dbSchema.reservations, compairData, {
            order_id: 1,
            price: 1,
            company_earn: 1,
            owner_earn: 1,
        }, (error, reservations) => {
            let monthEarning = LD.sumBy(reservations, column);
            monthEarning = parseFloat(monthEarning).toFixed(2)
            resolve(parseFloat(monthEarning));
            return;
        });
    })
}; 

const getCustomerReport = async(req, done) => {
    let user_type = ["both"]
    user_type.push(req.query.user_type)
    let columnAndValue = {
        status:'active',
        user_type: {$in: user_type}
    }
    let project = {
        _id: 0,
        customer_id: "$customer_id",
        name: "$name",
        user_type: "$user_type",
        data: "$resDetails",
        amount:""
    }
    if(req.query.start_date !='null' && req.query.end_date !='null'){
        project = {
            _id: 0,
            customer_id: "$customer_id",
            name: "$name",
            data: {
                $filter: {
                    input: "$resDetails",
                    as: "order",
                    cond: {
                        $and: [
                            { $gte: [ "$$order.created_at", new Date(req.query.start_date+'T00:00:00.000Z') ] },
                            { $lte: [ "$$order.created_at", new Date(req.query.end_date+'T23:59:59.000Z') ] },
                            //{ $eq: ["$$order.status", "completed"]}
                        ]
                    }
                }
            },
            amount:""
        }
    }
    let joinArr = []
    if(req.query.user_type == 'seeker'){
        joinArr = [{
            $lookup: {
                from: 'reservations',
                localField: 'customer_id',
                foreignField: 'customer_id',
                as: 'resDetails'
            }
        },  { 
            $match : columnAndValue
        }, { 
            $sort : {created_at:-1}
        }, {
            $project: project
        }];
    }
    else{
        joinArr = [{
            $lookup: {
                from: 'reservations',
                localField: 'customer_id',
                foreignField: 'owner_id',
                as: 'resDetails'
            }
        },  { 
            $match : columnAndValue
        }, { 
            $sort : {created_at:-1}
        }, {
            $project: project
        }];
    }
    console.log(columnAndValue)
    query.selectWithAndOne(dbConstants.dbSchema.settings,{},{default_currency:1}, function(error, settings) {
        query.joinWithAnd(dbConstants.dbSchema.customers, joinArr, (error, response) => {
            if (error) {
                logger('Error: can not get record.');
                done(errors.internalServer(true), null);
                return;
            }
            let arr = [];
            _.each(response, (elem) => {
                if(elem.data.length > 0){
                    elem.total_booking = elem.data.length
                    elem.amount = settings.default_currency+' '+parseFloat(LD.sumBy(elem.data, 'price')).toFixed(2)
                    arr.push(elem)
                }
            })
            done(null, arr)
        });
    });
};

const getPropertyReport = async(req, done) => {
    let columnAndValue = {
        status:'completed'
    }
    let project = {
        _id: 0,
        property_id: "$property_id",
        address: "$address",
        data: "$resDetails",
        amount:""
    }
    if(req.query.start_date !='null' && req.query.end_date !='null'){
        project = {
            _id: 0,
            property_id: "$property_id",
            address: "$address",
            data: {
                $filter: {
                    input: "$resDetails",
                    as: "order",
                    cond: {
                        $and: [
                            { $gte: [ "$$order.created_at", new Date(req.query.start_date+'T00:00:00.000Z') ] },
                            { $lte: [ "$$order.created_at", new Date(req.query.end_date+'T23:59:59.000Z') ] },
                            //{ $eq: ["$$order.status", "completed"]}
                        ]
                    }
                }
            },
            amount:""
        }
    }
    let joinArr = [{
        $lookup: {
            from: 'reservations',
            localField: 'property_id',
            foreignField: 'property_id',
            as: 'resDetails'
        }
    },  { 
        $match : columnAndValue
    }, { 
        $sort : {created_at:-1}
    }, {
        $project: project
    }];
    query.selectWithAndOne(dbConstants.dbSchema.settings,{},{default_currency:1}, function(error, settings) {
        query.joinWithAnd(dbConstants.dbSchema.properties, joinArr, (error, response) => {
            if (error) {
                logger('Error: can not get record.');
                done(errors.internalServer(true), null);
                return;
            }
            let arr = []
            _.each(response, (elem) => {
                if(elem.data.length > 0){
                    elem.total_booking = elem.data.length
                    elem.amount = settings.default_currency+' '+parseFloat(LD.sumBy(elem.data, 'price')).toFixed(2)
                    arr.push(elem)
                }
            })
            done(null, arr)
        });
    });
};

const getSalesReport = async(req, done) => {
    var getDaysArray = function(start, end) {
        for(var arr=[],dt=new Date(start); dt<=end; dt.setDate(dt.getDate()+1)){
            arr.push(moment(new Date(dt)).format('YYYY-MM-DD'));
        }
        return arr;
    };
    var dateArr = getDaysArray(new Date(req.query.start_date),new Date(req.query.end_date));
    let arr = [];
    query.selectWithAndOne(dbConstants.dbSchema.settings,{},{default_currency:1}, async function(error, settings) {
        await forEach(dateArr, async(date) => {
            let columnAndValue = {}
            columnAndValue = {
                completed_date: {
                    $lte: new Date(date+'T23:59:59.000Z'),
                    $gte: new Date(date+'T00:00:00.000Z')
                }
            }
            let response = await queryApi.selectWithAnd(dbConstants.dbSchema.reservations, columnAndValue, { _id: 0, price:1, } );
            let amount = 0;
            _.each(response, (elem) => {
                amount += parseFloat(elem.price)
            })
            arr.push({
                date: date,
                amount: settings.default_currency+' '+parseFloat(amount).toFixed(2)
            })
        });
        arr.sort((a, b) => a.date !== b.date ? a.date < b.date ? -1 : 1 : 0);
        let graphHeader = _.pluck(arr, 'date')
        let graphValue = _.pluck(arr, 'amount')
        let newarr = [];
        _.each(graphValue, (elem) => {
            elem = elem.replace( /^\D+/g, '');
            newarr.push(parseFloat(elem))
        })
        done(null, {data: arr, graphHeader:graphHeader, graphValue:newarr})
    });
};


module.exports = {
	getCustomerReport,
    getPropertyReport,
    getSalesReport,
    totalSales,
    expectedSales,
    totalEarnings,
    expectedEarnings,
    totalPayouts,
};