'use strict';

const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const dbConstants = require('./../../constants/db-constants');
const query = require('./../../utils/query-creator');
let async = require('async');
let _ = require('underscore');
const Role = require('./../../models/role');
const Access_right = require('./../../models/access-right');

const get = function(req,done){
	let columnAndValue={}
	if(req.query.role_id){
		columnAndValue.role_id = req.query.role_id
	}
    if(req.query.status){
        columnAndValue.status = req.query.status
    }
	query.selectWithAndFilter(dbConstants.dbSchema.roles, columnAndValue, {
        _id: 0,
    }, {created_at:-1}, {}, (error, roles) => {
    	if (error) {
            logger('Error: can not get ', dbConstants.dbSchema.roles);
            done(errors.internalServer(true), null);
            return;
        }
        done(null,roles)
    });
};

const create = function(requestParam,done){
	query.insertSingle(dbConstants.dbSchema.roles,requestParam,function (error, role) {
		if (error) {
			logger('Error: can not create role');
			done(error, null);
			return;
		}
        accessRightInsert(role.role_id, done);
		//done(null, role);
	});
};

const update = function(requestParam,done){
	query.updateSingle(dbConstants.dbSchema.roles,requestParam, { 'role_id':requestParam.role_id},function (error, role) {
		if (error) {
			logger('Error: can not update role');
			done(error, null);
			return;
		}
        accessRightInsert(requestParam.role_id, done);
		//done(null, role);
	});
};

const action  = (requestParam, done) => {
 	if (requestParam['type']=="delete") {
        query.removeMultiple(dbConstants.dbSchema.roles, {
            'role_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not delete ');
                done(error, null);
                return;
            }
            query.removeMultiple(dbConstants.dbSchema.access_rights, {
                'role_id': {
                    $in: requestParam['ids']
                }
            }, function(error, data) {
                if (error) {
                    logger('Error: can not delete ');
                    done(error, null);
                    return;
                }
                done(null, data);
            });
        });        
    }
    else
    {
        let columnsToUpdate = {
            status: requestParam['type']
        };
       query.updateMultiple(dbConstants.dbSchema.roles, columnsToUpdate, {
            'role_id': {
                $in: requestParam['ids']
            }
        }, function(error, data) {
            if (error) {
                logger('Error: can not update ');
                done(error, null);
                return;
            }
            done(null, data);
        });  
    }
};

const accessRightInsert = (role_id, done) => {
    query.selectWithAndFilter(dbConstants.dbSchema.modules, {}, {
        _id: 0,
        title:1
    }, {order_no:1}, {}, (error, response) => {
        let obj = {}
        _.each(response, (elem) => {
            obj[elem.title] = {
                is_view: true,
                is_add: true,
                is_edit: true,
                is_delete: true,
            }
        });
        let insertObj = {
            role_id: role_id,
            module: obj
        }
        query.selectWithAndOne(dbConstants.dbSchema.access_rights, {role_id:role_id}, {
            role_id: 1
        }, async(error, exists) => {
            if(exists){
                query.updateSingle(dbConstants.dbSchema.access_rights, insertObj, { 'role_id':role_id},function (error, role) {
                    done(null, {role_id:role_id})
                });
            }   
            else{
                query.insertSingle(dbConstants.dbSchema.access_rights, insertObj ,function (error, role) {
                    done(null, {role_id:role_id})
                });
            }
        });
    });
};

module.exports = {
	get,
	create,
	action,
	update,
    accessRightInsert
};