// grab the things we need
var mongoose = require('mongoose');
var Schema = mongoose.Schema;
const idGenerator = require('./../utils/id-generator');

// create a schema
var propertySchema = new Schema({
    property_id: {
        type: String,
        default:''
    },
    customer_id: {
        type: String,
        default:''
    },
    address: {
        type: String,
        default:''
    },
    latitude: {
        type: Number,
        default: 0
    },
    longitude: {
        type: Number,
        default: 0
    },
    grace_period: {
        type: Number,
        default:0
    },
    location: {
        type: {
            type: String,
            default: "Point",
        },
        coordinates: {
            type: [Number],
            default: [0, 0],
        },
    },
    property_type: {
        type: String,
        enum: ['public', 'private']
    },
    parking_spots: {
        type: Number,
        default: 0
    },
    entrance_property: {
        type: String,
        default:''
    },
    entrance_latitude: {
        type: Number,
        default: 0
    },
    entrance_longitude: {
        type: Number,
        default: 0
    },
    contact_details: {
        type: Object,
        default:{}
    },
    status: {
        type: String,
        enum: ['completed', 'incomplete'],
        default:'incomplete'
    },
    discoverable_status: {
        type: String,
        enum: ['on', 'off'],
        default:'on'
    },
    geo_tag: {
        type: Object,
        default:{}
    },
    parking_size: {
        type: Object,
        default:{}
    },
    parking_spot_type_id: {
        type: String,
        default:''
    },
    is_apply_parking_spot_type: {
        type: Boolean,
        default:false
    },
    parking_spot_coverage_id: {
        type: String,
        default:''
    },
    is_apply_parking_spot_coverage: {
        type: Boolean,
        default:false
    },
    parking_spot_terrain_id: {
        type: String,
        default:''
    },
    is_apply_parking_spot_terrain: {
        type: Boolean,
        default:false
    },
    parking_spot_amenity_id: {
        type: Array,
        default:[]
    },
    is_apply_parking_spot_amenity: {
        type: Boolean,
        default:false
    },
    parking_spot_description: {
        type: String,
        default:''
    },
    is_apply_parking_spot_description: {
        type: Boolean,
        default:false
    },
    is_apply_parking_spot_photos: {
        type: Boolean,
        default:false
    },
    parking_spot_photos: {
        type: Array,
        default:[]
    },
    is_specific_date_range: {
        type: Boolean,
        default:false
    },
    from_date_range: {
        type: Date,
        default: Date.now
    },
    to_date_range: {
        type: Date,
        default: Date.now
    },
    available_days: {
        type: Array,
        default:[]
    },
    from_reservation_time: {
        type: String,
        default:''
    },
    to_reservation_time: {
        type: String,
        default:''
    },
    from_entry_time: {
        type: String,
        default:''
    },
    to_entry_time: {
        type: String,
        default:''
    },
    is_auto_activate: {
        type: String,
        enum: ["disabled", "enabled"],
        default:"disabled"
    },
    distance_from_parking_spot: {
        type: Number,
        default: 0
    },
    is_parking_availability: {
        type: Boolean,
        default:false
    },
    is_apply_access_instruction: {
        type: Boolean,
        default:false
    },
    access_instruction_id: {
        type: String,
        default:''
    },
    private_instruction: {
        type: String,
        default:''
    },
    pricing: {
        type: Object,
        default:{}
    },
    is_apply_pricing: {
        type: Boolean,
        default:false
    },
    completed_spots: {
        type: Number,
        default: 0
    },
    is_added_geo_tag: {
        type: Boolean,
        default:false
    },
    is_added_parking_size: {
        type: Boolean,
        default:false
    },
    is_added_amenities: {
        type: Boolean,
        default:false
    },
    is_added_description_and_photos: {
        type: Boolean,
        default:false
    },
    is_added_availability: {
        type: Boolean,
        default:false
    },
    is_added_access_instruction: {
        type: Boolean,
        default:false
    },
    is_added_pricing: {
        type: Boolean,
        default:false
    },
    is_added_grace_period: {
        type: Boolean,
        default:false
    },
    created_at: {
        type: Date,
        default: Date.now
    },
    updated_at: {
        type: Date,
        default: Date.now
    }
});

propertySchema.index({ location: '2dsphere' });

// // Execute before each user.save() call
propertySchema.pre('save', function(callback) {
    idGenerator.generateId('properties', 'property_id', 'PRO', (err, ID) => {
        this.property_id = ID;
        callback();
    });
});

var Property = mongoose.model('Property', propertySchema);
module.exports = Property;
