'use strict';

const config = require('./../../config');
const responseCodes = require('./../../helpers/response-codes');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors-api');
const labels = require('./../../utils/labels.json');
const express = require('express');
const router = express.Router();
const propertyHandler = require('./../../model_handlers/api/property-handler');
const _ = require('underscore');

/*
    Name : Upload or Delete Package Image
    Purpose : Upload or Delete Package Image
    Original Author : Gaurav Patel
    Created At : 17th Mar 2021
*/
router.post('/upload-image', async(req, res) => {
    try {
        console.log("upload-image api called here")
        console.log("req.body log here:- ",req.body)
        if (!req.body.code || !req.body.action) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        if(req.body.action == 'upload'){
            if (!req.files.photo) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.action == 'delete'){
            if (!req.body.photo) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        let response = await propertyHandler.uploadImage(req.body, req);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : List
    Purpose : List
    Original Author : Gaurav Patel
    Created At : 12th March 2021
*/
router.get('/list', async(req, res) => {
    try {
        if (_.isEmpty(req.query) || !req.query.code || !req.query.customer_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await propertyHandler.list(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Add one
    Purpose : Add one
    Original Author : Gaurav Patel
    Created At : 12th March 2021
*/
router.post('/add', async(req, res) => {
    try {
        if (!req.body.code || !req.body.customer_id || !req.body.address || !req.body.property_type || !req.body.parking_spots || !req.body.entrance_property || !req.body.contact_details) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }

        let response = await propertyHandler.add(req.body);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : update-property
    Purpose : update-property
    Original Author : Gaurav Patel
    Created At : 12th March 2021
*/
router.post('/update', async(req, res) => {
    try {
        console.log("property update api called")
        req.body.time_zone = config.time_zone
        if(req.headers.time_zone){
            req.body.time_zone = req.headers.time_zone
        }
        if (!req.body.code || !req.body.customer_id || !req.body.property_id || !req.body.method) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        if(req.body.method == 'geo_tag'){
            if (!req.body.latitude || !req.body.longitude || !req.body.address) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.method == 'parking_size'){
            if (!req.body.vehicle_type_id || !req.body.is_apply) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.method == 'amenities'){
            if (!req.body.parking_spot_type_id || !req.body.parking_spot_coverage_id || !req.body.parking_spot_terrain_id) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
            if (!req.body.is_apply_parking_spot_type || !req.body.is_apply_parking_spot_coverage || !req.body.is_apply_parking_spot_terrain || !req.body.is_apply_parking_spot_amenity) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.method == 'description_and_photos'){
            if (!req.body.parking_spot_description || !req.body.is_apply_parking_spot_description || !req.body.parking_spot_photos || !req.body.is_apply_parking_spot_photos) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.method == 'availability'){
            if (!req.body.grace_period || !req.body.is_specific_date_range || !req.body.available_days || !req.body.from_reservation_time || !req.body.to_reservation_time || !req.body.from_entry_time || !req.body.to_entry_time) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
            if (!req.body.is_parking_availability || !req.body.is_auto_activate) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
            if (req.body.is_specific_date_range == false) {
                if (!req.body.from_date_range || !req.body.to_date_range) {
                    jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                    return;
                }
            }
            if (req.body.is_auto_activate == 'enabled') {
                if (!req.body.distance_from_parking_spot) {
                    jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                    return;
                }
            }
        }
        if(req.body.method == 'access_instruction'){
            if (!req.body.access_instruction_id || !req.body.is_apply_access_instruction) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        if(req.body.method == 'pricing'){
            if (!req.body.is_apply_pricing || !req.body.pricing) {
                jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
                return;
            }
        }
        // console.log()
        let response = await propertyHandler.update(req.body, req);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

module.exports = router;