'use strict';

let _ = require('underscore');
const express = require('express');
const config = require('./../../config');
const responseCodes = require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors-api');
const router = express.Router();
const socketHandler = require('./../../model_handlers/api/socket-handler');

/*
    Name : List
    Purpose : List
    Original Author : Gaurav Patel
    Created At : 3rd May 2021
*/
router.get('/chat-list', async(req, res) => {
    try {
        if (_.isEmpty(req.query) || !req.query.code || !req.query.room || !req.query.user_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await socketHandler.list(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

/*
    Name : Upload or Delete Package Image
    Purpose : Upload or Delete Package Image
    Original Author : Gaurav Patel
    Created At : 3rd May 2021
*/
router.post('/upload-image', async(req, res) => {
    try {
        if (!req.body.code || !req.files.photo) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.body.code), null);
            return;
        }
        let response = await socketHandler.uploadImage(req.body, req);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.body.code), null);
            return;
        }
    }
});

/*
    Name : read chat msg
    Purpose : read chat msg
    Original Author : Gaurav Patel
    Created At : 8th June 2021
*/
router.get('/read-msg', async(req, res) => {
    try {
        if (_.isEmpty(req.query) || !req.query.code || !req.query.room || !req.query.user_id) {
            jsonResponse(res, responseCodes.BadRequest, errors.missingParameters(true, req.query.code), null);
            return;
        }

        let response = await socketHandler.readMsg(req.query);
        jsonResponse(res, responseCodes.OK, errors.noError(), response);
    } catch (error) {
        try {
            jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
            return;
        } catch (error) {
            jsonResponse(res, responseCodes.InternalServer, errors.internalServer(true, req.query.code), null);
            return;
        }
    }
});

module.exports = router;