'use strict';

const responseCodes = require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const express = require('express');
const router = express.Router();
const _ = require('underscore');
const backupHandler = require('./../../model_handlers/backend/backup-handler');

router.post('/delete-backup', function(req, res) {
   backupHandler.deleteBackup(req.body, (error, response) => {
       if (error) {
           jsonResponse( res, error.code, errors.formatErrorForWire(error), null);
           return;
       }
       jsonResponse(res, responseCodes.OK, errors.noError(), response);
   });
});

router.post('/restore-backup', function(req, res) {
   backupHandler.restoreBackup(req.body, (error, response) => {
       if (error) {
           jsonResponse( res, error.code, errors.formatErrorForWire(error), null);
           return;
       }
       jsonResponse( res, responseCodes.OK, errors.noError(), response);
   });
});

router.post('/create-new-backup', function(req, res) {
   backupHandler.createNewBackup((error, response) => {
       if (error) {
           jsonResponse( res, error.code, errors.formatErrorForWire(error), null);
           return;
       }
       jsonResponse( res, responseCodes.OK, errors.noError(), response);
   });
});

router.post('/list-db', function(req, res) {
   backupHandler.listDB((error, response) => {
       if (error) {
           jsonResponse( res, error.code, errors.formatErrorForWire(error), null);
           return;
       }
       jsonResponse( res, responseCodes.OK, errors.noError(), response);
   });
});

module.exports = router;