'use strict';

const responseCodes =  require('./../../helpers/response-codes');
const logger = require('./../../utils/logger');
const jsonResponse = require('./../../utils/json-response');
const errors = require('./../../utils/dz-errors');
const express = require('express');
const router = express.Router();
const reportHandler = require('./../../model_handlers/backend/report-handler');

router.get('/total-sales', function (req, res) {
	reportHandler.totalSales(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/expected-sales', function (req, res) {
	reportHandler.expectedSales(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/total-earnings', function (req, res) {
	reportHandler.totalEarnings(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/expected-earnings', function (req, res) {
	reportHandler.expectedEarnings(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/total-payouts', function (req, res) {
	reportHandler.totalPayouts(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/get-customer-report', function (req, res) {
	reportHandler.getCustomerReport(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

router.get('/get-property-report', function (req, res) {
	reportHandler.getPropertyReport(req,function (error, labels) {
		if (error) {
			jsonResponse(res, error.code, errors.formatErrorForWire(error), null);
			return;
		}
		jsonResponse(res, responseCodes.OK, errors.noError(), labels);
	});
});

module.exports = router;